import sys
from _typeshed import FileDescriptorOrPath
from collections.abc import Callable, Generator, Iterable, Sequence
from re import Pattern
from token import *
from typing import Any, Final, NamedTuple, TextIO, type_check_only
from typing_extensions import TypeAlias, disjoint_base

if sys.version_info < (3, 12):
    # Avoid double assignment to Final name by imports, which pyright objects to.
    # EXACT_TOKEN_TYPES is already defined by 'from token import *' above
    # in Python 3.12+.
    from token import EXACT_TOKEN_TYPES as EXACT_TOKEN_TYPES

__all__ = [
    "AMPER",
    "AMPEREQUAL",
    "AT",
    "ATEQUAL",
    "CIRCUMFLEX",
    "CIRCUMFLEXEQUAL",
    "COLON",
    "COLONEQUAL",
    "COMMA",
    "COMMENT",
    "DEDENT",
    "DOT",
    "DOUBLESLASH",
    "DOUBLESLASHEQUAL",
    "DOUBLESTAR",
    "DOUBLESTAREQUAL",
    "ELLIPSIS",
    "ENCODING",
    "ENDMARKER",
    "EQEQUAL",
    "EQUAL",
    "ERRORTOKEN",
    "GREATER",
    "GREATEREQUAL",
    "INDENT",
    "ISEOF",
    "ISNONTERMINAL",
    "ISTERMINAL",
    "LBRACE",
    "LEFTSHIFT",
    "LEFTSHIFTEQUAL",
    "LESS",
    "LESSEQUAL",
    "LPAR",
    "LSQB",
    "MINEQUAL",
    "MINUS",
    "NAME",
    "NEWLINE",
    "NL",
    "NOTEQUAL",
    "NT_OFFSET",
    "NUMBER",
    "N_TOKENS",
    "OP",
    "PERCENT",
    "PERCENTEQUAL",
    "PLUS",
    "PLUSEQUAL",
    "RARROW",
    "RBRACE",
    "RIGHTSHIFT",
    "RIGHTSHIFTEQUAL",
    "RPAR",
    "RSQB",
    "SEMI",
    "SLASH",
    "SLASHEQUAL",
    "STAR",
    "STAREQUAL",
    "STRING",
    "TILDE",
    "TYPE_COMMENT",
    "TYPE_IGNORE",
    "TokenInfo",
    "VBAR",
    "VBAREQUAL",
    "detect_encoding",
    "generate_tokens",
    "tok_name",
    "tokenize",
    "untokenize",
]
if sys.version_info < (3, 13):
    __all__ += ["ASYNC", "AWAIT"]

if sys.version_info >= (3, 10):
    __all__ += ["SOFT_KEYWORD"]

if sys.version_info >= (3, 12):
    __all__ += ["EXCLAMATION", "FSTRING_END", "FSTRING_MIDDLE", "FSTRING_START", "EXACT_TOKEN_TYPES"]

if sys.version_info >= (3, 13):
    __all__ += ["TokenError", "open"]

if sys.version_info >= (3, 14):
    __all__ += ["TSTRING_START", "TSTRING_MIDDLE", "TSTRING_END"]

cookie_re: Final[Pattern[str]]
blank_re: Final[Pattern[bytes]]

_Position: TypeAlias = tuple[int, int]

# This class is not exposed. It calls itself tokenize.TokenInfo.
@type_check_only
class _TokenInfo(NamedTuple):
    type: int
    string: str
    start: _Position
    end: _Position
    line: str

if sys.version_info >= (3, 12):
    class TokenInfo(_TokenInfo):
        @property
        def exact_type(self) -> int: ...

else:
    @disjoint_base
    class TokenInfo(_TokenInfo):
        @property
        def exact_type(self) -> int: ...

# Backwards compatible tokens can be sequences of a shorter length too
_Token: TypeAlias = TokenInfo | Sequence[int | str | _Position]

class TokenError(Exception): ...

if sys.version_info < (3, 13):
    class StopTokenizing(Exception): ...  # undocumented

class Untokenizer:
    tokens: list[str]
    prev_row: int
    prev_col: int
    encoding: str | None
    def add_whitespace(self, start: _Position) -> None: ...
    if sys.version_info >= (3, 12):
        def add_backslash_continuation(self, start: _Position) -> None: ...

    def untokenize(self, iterable: Iterable[_Token]) -> str: ...
    def compat(self, token: Sequence[int | str], iterable: Iterable[_Token]) -> None: ...
    if sys.version_info >= (3, 12):
        def escape_brackets(self, token: str) -> str: ...

# Returns str, unless the ENCODING token is present, in which case it returns bytes.
def untokenize(iterable: Iterable[_Token]) -> str | Any: ...
def detect_encoding(readline: Callable[[], bytes | bytearray]) -> tuple[str, Sequence[bytes]]: ...
def tokenize(readline: Callable[[], bytes | bytearray]) -> Generator[TokenInfo, None, None]: ...
def generate_tokens(readline: Callable[[], str]) -> Generator[TokenInfo, None, None]: ...
def open(filename: FileDescriptorOrPath) -> TextIO: ...
def group(*choices: str) -> str: ...  # undocumented
def any(*choices: str) -> str: ...  # undocumented
def maybe(*choices: str) -> str: ...  # undocumented

Whitespace: Final[str]  # undocumented
Comment: Final[str]  # undocumented
Ignore: Final[str]  # undocumented
Name: Final[str]  # undocumented

Hexnumber: Final[str]  # undocumented
Binnumber: Final[str]  # undocumented
Octnumber: Final[str]  # undocumented
Decnumber: Final[str]  # undocumented
Intnumber: Final[str]  # undocumented
Exponent: Final[str]  # undocumented
Pointfloat: Final[str]  # undocumented
Expfloat: Final[str]  # undocumented
Floatnumber: Final[str]  # undocumented
Imagnumber: Final[str]  # undocumented
Number: Final[str]  # undocumented

def _all_string_prefixes() -> set[str]: ...  # undocumented

StringPrefix: Final[str]  # undocumented

Single: Final[str]  # undocumented
Double: Final[str]  # undocumented
Single3: Final[str]  # undocumented
Double3: Final[str]  # undocumented
Triple: Final[str]  # undocumented
String: Final[str]  # undocumented

Special: Final[str]  # undocumented
Funny: Final[str]  # undocumented

PlainToken: Final[str]  # undocumented
Token: Final[str]  # undocumented

ContStr: Final[str]  # undocumented
PseudoExtras: Final[str]  # undocumented
PseudoToken: Final[str]  # undocumented

endpats: Final[dict[str, str]]  # undocumented
single_quoted: Final[set[str]]  # undocumented
triple_quoted: Final[set[str]]  # undocumented

tabsize: Final = 8  # undocumented



#!/usr/bin/env python3
"""
Script to update the typeshed-fallback folder with the latest files from
the typeshed repository (https://github.com/python/typeshed).

This script:
1. Clones/downloads the typeshed repository to a temporary directory
2. Copies the stdlib/ and stubs/ folders to typeshed-fallback
3. Copies the LICENSE and README.md files
4. Updates commit.txt with the current commit hash
"""

import argparse
import os
import shutil
import subprocess
import sys
import tempfile
from pathlib import Path


def get_script_dir() -> Path:
    """Get the directory where this script is located."""
    return Path(__file__).parent.resolve()


def get_typeshed_fallback_dir() -> Path:
    """Get the path to the typeshed-fallback directory."""
    script_dir = get_script_dir()
    return script_dir.parent / "packages" / "pyright-internal" / "typeshed-fallback"


def run_git_command(args: list[str], cwd: Path) -> subprocess.CompletedProcess:
    """Run a git command and return the result."""
    return subprocess.run(
        ["git"] + args,
        cwd=cwd,
        capture_output=True,
        text=True,
        check=True,
    )


def clone_typeshed(target_dir: Path, commit: str | None = None) -> str:
    """
    Clone the typeshed repository to the target directory.
    
    Args:
        target_dir: Directory to clone into
        commit: Optional specific commit hash to checkout
        
    Returns:
        The commit hash that was checked out
    """
    typeshed_url = "https://github.com/python/typeshed.git"
    
    print(f"Cloning typeshed repository to {target_dir}...")
    
    # Clone with depth 1 for faster download (unless we need a specific commit)
    if commit:
        # Full clone needed for specific commit
        run_git_command(["clone", typeshed_url, str(target_dir)], cwd=target_dir.parent)
        run_git_command(["checkout", commit], cwd=target_dir)
    else:
        # Shallow clone for latest
        run_git_command(["clone", "--depth", "1", typeshed_url, str(target_dir)], cwd=target_dir.parent)
    
    # Get the current commit hash
    result = run_git_command(["rev-parse", "HEAD"], cwd=target_dir)
    commit_hash = result.stdout.strip()
    
    print(f"Checked out commit: {commit_hash}")
    return commit_hash


def remove_directory_contents(dir_path: Path) -> None:
    """Remove all contents of a directory but keep the directory itself."""
    if dir_path.exists():
        shutil.rmtree(dir_path)
    dir_path.mkdir(parents=True, exist_ok=True)


def should_copy_file(file_path: Path) -> bool:
    """Check if a file should be copied based on its extension or name."""
    allowed_extensions = {".pyi", ".toml"}
    allowed_names = {"VERSIONS"}
    return file_path.suffix.lower() in allowed_extensions or file_path.name in allowed_names


def is_in_excluded_folder(file_path: Path, base_folder: Path) -> bool:
    """Check if the file is inside a folder that starts with '@'."""
    rel_path = file_path.relative_to(base_folder)
    for part in rel_path.parts:
        if part.startswith("@"):
            return True
    return False


def copy_tree_filtered(src_folder: Path, dst_folder: Path) -> None:
    """
    Copy a directory tree, only including .pyi and VERSIONS files.
    Skips any folder starting with '@'.
    
    Args:
        src_folder: Source directory
        dst_folder: Destination directory
    """
    for src_path in src_folder.rglob("*"):
        if src_path.is_file() and should_copy_file(src_path):
            # Skip files in folders starting with '@'
            if is_in_excluded_folder(src_path, src_folder):
                continue
            
            # Calculate relative path and destination
            rel_path = src_path.relative_to(src_folder)
            dst_path = dst_folder / rel_path
            
            # Create parent directories if needed
            dst_path.parent.mkdir(parents=True, exist_ok=True)
            
            # Copy the file
            shutil.copy2(src_path, dst_path)


def copy_typeshed_files(source_dir: Path, dest_dir: Path) -> None:
    """
    Copy the relevant typeshed files to the destination directory.
    Only .pyi and VERSIONS files are copied from folders.
    
    Args:
        source_dir: The cloned typeshed repository directory
        dest_dir: The typeshed-fallback directory
    """
    # Folders to copy
    folders_to_copy = ["stdlib", "stubs"]
    
    # Copy folders (only .py and .pyi files)
    for folder in folders_to_copy:
        src_folder = source_dir / folder
        dst_folder = dest_dir / folder
        
        if not src_folder.exists():
            print(f"Warning: Source folder {src_folder} does not exist, skipping...")
            continue
            
        print(f"Copying {folder}/ (only .pyi and VERSIONS files)...")
        
        # Remove existing folder contents
        remove_directory_contents(dst_folder)
        
        # Copy the folder with filtering
        copy_tree_filtered(src_folder, dst_folder)

    # Files to copy
    files_to_copy = ["LICENSE", "README.md"]
    
    # Copy files
    for file in files_to_copy:
        src_file = source_dir / file
        dst_file = dest_dir / file
        
        if not src_file.exists():
            print(f"Warning: Source file {src_file} does not exist, skipping...")
            continue
            
        print(f"Copying {file}...")
        shutil.copy2(src_file, dst_file)


def update_commit_file(dest_dir: Path, commit_hash: str) -> None:
    """Update the commit.txt file with the new commit hash."""
    commit_file = dest_dir / "commit.txt"
    print(f"Updating commit.txt with {commit_hash}...")
    commit_file.write_text(commit_hash + "\n")


def main() -> int:
    parser = argparse.ArgumentParser(
        description="Update typeshed-fallback with the latest typeshed files"
    )
    parser.add_argument(
        "--commit",
        "-c",
        type=str,
        default=None,
        help="Specific commit hash to checkout (default: latest main branch)",
    )
    parser.add_argument(
        "--dry-run",
        "-n",
        action="store_true",
        help="Show what would be done without making changes",
    )
    
    args = parser.parse_args()
    
    typeshed_fallback_dir = get_typeshed_fallback_dir()
    
    if not typeshed_fallback_dir.exists():
        print(f"Error: typeshed-fallback directory not found at {typeshed_fallback_dir}")
        return 1
    
    print(f"Typeshed fallback directory: {typeshed_fallback_dir}")
    
    if args.dry_run:
        print("\n*** DRY RUN - No changes will be made ***\n")
        print("Would perform the following actions:")
        print("  1. Clone typeshed repository to a temporary directory")
        if args.commit:
            print(f"  2. Checkout commit: {args.commit}")
        else:
            print("  2. Use latest commit from main branch")
        print("  3. Copy stdlib/ folder (only .pyi and VERSIONS files)")
        print("  4. Copy stubs/ folder (only .pyi and VERSIONS files)")
        print("  5. Copy LICENSE file")
        print("  6. Copy README.md file")
        print("  7. Update commit.txt with new commit hash")
        return 0
    
    # Create a temporary directory for cloning
    with tempfile.TemporaryDirectory() as temp_dir:
        temp_path = Path(temp_dir)
        typeshed_clone_dir = temp_path / "typeshed"
        
        try:
            # Clone typeshed
            commit_hash = clone_typeshed(typeshed_clone_dir, args.commit)
            
            # Copy files
            copy_typeshed_files(typeshed_clone_dir, typeshed_fallback_dir)
            
            # Update commit.txt
            update_commit_file(typeshed_fallback_dir, commit_hash)
            
            print("\nTypeshed update complete!")
            print(f"Updated to commit: {commit_hash}")
            
        except subprocess.CalledProcessError as e:
            print(f"Error running git command: {e}")
            print(f"stdout: {e.stdout}")
            print(f"stderr: {e.stderr}")
            return 1
        except Exception as e:
            print(f"Error: {e}")
            return 1
    
    return 0


if __name__ == "__main__":
    sys.exit(main())



import sys
from collections.abc import Iterable, Iterator
from typing import Any, Final, Generic, Literal, TypeVar, overload
from typing_extensions import Self, TypeAlias

# Undocumented length constants
IPV4LENGTH: Final = 32
IPV6LENGTH: Final = 128

_A = TypeVar("_A", IPv4Address, IPv6Address)
_N = TypeVar("_N", IPv4Network, IPv6Network)

_RawIPAddress: TypeAlias = int | str | bytes | IPv4Address | IPv6Address
_RawNetworkPart: TypeAlias = IPv4Network | IPv6Network | IPv4Interface | IPv6Interface

def ip_address(address: _RawIPAddress) -> IPv4Address | IPv6Address: ...
def ip_network(
    address: _RawIPAddress | _RawNetworkPart | tuple[_RawIPAddress] | tuple[_RawIPAddress, int], strict: bool = True
) -> IPv4Network | IPv6Network: ...
def ip_interface(
    address: _RawIPAddress | _RawNetworkPart | tuple[_RawIPAddress] | tuple[_RawIPAddress, int],
) -> IPv4Interface | IPv6Interface: ...

class _IPAddressBase:
    __slots__ = ()
    @property
    def compressed(self) -> str:
        """Return the shorthand version of the IP address as a string."""
        ...
    @property
    def exploded(self) -> str:
        """Return the longhand version of the IP address as a string."""
        ...
    @property
    def reverse_pointer(self) -> str:
        """
        The name of the reverse DNS pointer for the IP address, e.g.:
        >>> ipaddress.ip_address("127.0.0.1").reverse_pointer
        '1.0.0.127.in-addr.arpa'
        >>> ipaddress.ip_address("2001:db8::1").reverse_pointer
        '1.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.8.b.d.0.1.0.0.2.ip6.arpa'
        """
        ...
    if sys.version_info < (3, 14):
        @property
        def version(self) -> int: ...

class _BaseAddress(_IPAddressBase):
    __slots__ = ()
    def __add__(self, other: int) -> Self: ...
    def __hash__(self) -> int: ...
    def __int__(self) -> int: ...
    def __sub__(self, other: int) -> Self: ...
    def __format__(self, fmt: str) -> str: ...
    def __eq__(self, other: object) -> bool: ...
    def __lt__(self, other: Self) -> bool: ...
    if sys.version_info >= (3, 11):
        def __ge__(self, other: Self) -> bool: ...
        def __gt__(self, other: Self) -> bool: ...
        def __le__(self, other: Self) -> bool: ...
    else:
        def __ge__(self, other: Self, NotImplemented: Any = ...) -> bool: ...
        def __gt__(self, other: Self, NotImplemented: Any = ...) -> bool: ...
        def __le__(self, other: Self, NotImplemented: Any = ...) -> bool: ...

class _BaseNetwork(_IPAddressBase, Generic[_A]):
    network_address: _A
    netmask: _A
    def __contains__(self, other: Any) -> bool: ...
    def __getitem__(self, n: int) -> _A: ...
    def __iter__(self) -> Iterator[_A]: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __lt__(self, other: Self) -> bool: ...
    if sys.version_info >= (3, 11):
        def __ge__(self, other: Self) -> bool: ...
        def __gt__(self, other: Self) -> bool: ...
        def __le__(self, other: Self) -> bool: ...
    else:
        def __ge__(self, other: Self, NotImplemented: Any = ...) -> bool: ...
        def __gt__(self, other: Self, NotImplemented: Any = ...) -> bool: ...
        def __le__(self, other: Self, NotImplemented: Any = ...) -> bool: ...

    def address_exclude(self, other: Self) -> Iterator[Self]: ...
    @property
    def broadcast_address(self) -> _A: ...
    def compare_networks(self, other: Self) -> int: ...
    def hosts(self) -> Iterator[_A]: ...
    @property
    def is_global(self) -> bool:
        """
        Test if this address is allocated for public networks.

        Returns:
            A boolean, True if the address is not reserved per
            iana-ipv4-special-registry or iana-ipv6-special-registry.
        """
        ...
    @property
    def is_link_local(self) -> bool:
        """
        Test if the address is reserved for link-local.

        Returns:
            A boolean, True if the address is reserved per RFC 4291.
        """
        ...
    @property
    def is_loopback(self) -> bool:
        """
        Test if the address is a loopback address.

        Returns:
            A boolean, True if the address is a loopback address as defined in
            RFC 2373 2.5.3.
        """
        ...
    @property
    def is_multicast(self) -> bool:
        """
        Test if the address is reserved for multicast use.

        Returns:
            A boolean, True if the address is a multicast address.
            See RFC 2373 2.7 for details.
        """
        ...
    @property
    def is_private(self) -> bool:
        """
        Test if this network belongs to a private range.

        Returns:
            A boolean, True if the network is reserved per
            iana-ipv4-special-registry or iana-ipv6-special-registry.
        """
        ...
    @property
    def is_reserved(self) -> bool:
        """
        Test if the address is otherwise IETF reserved.

        Returns:
            A boolean, True if the address is within one of the
            reserved IPv6 Network ranges.
        """
        ...
    @property
    def is_unspecified(self) -> bool:
        """
        Test if the address is unspecified.

        Returns:
            A boolean, True if this is the unspecified address as defined in
            RFC 2373 2.5.2.
        """
        ...
    @property
    def num_addresses(self) -> int:
        """Number of hosts in the current subnet."""
        ...
    def overlaps(self, other: _BaseNetwork[IPv4Address] | _BaseNetwork[IPv6Address]) -> bool: ...
    @property
    def prefixlen(self) -> int: ...
    def subnet_of(self, other: Self) -> bool: ...
    def supernet_of(self, other: Self) -> bool: ...
    def subnets(self, prefixlen_diff: int = 1, new_prefix: int | None = None) -> Iterator[Self]: ...
    def supernet(self, prefixlen_diff: int = 1, new_prefix: int | None = None) -> Self: ...
    @property
    def with_hostmask(self) -> str: ...
    @property
    def with_netmask(self) -> str: ...
    @property
    def with_prefixlen(self) -> str: ...
    @property
    def hostmask(self) -> _A: ...

class _BaseV4:
    __slots__ = ()
    if sys.version_info >= (3, 14):
        version: Final = 4
        max_prefixlen: Final = 32
    else:
        @property
        def version(self) -> Literal[4]: ...
        @property
        def max_prefixlen(self) -> Literal[32]: ...

class IPv4Address(_BaseV4, _BaseAddress):
    __slots__ = ("_ip", "__weakref__")
    def __init__(self, address: object) -> None: ...
    @property
    def is_global(self) -> bool:
        """
        ``True`` if the address is defined as globally reachable by
        iana-ipv4-special-registry_ (for IPv4) or iana-ipv6-special-registry_
        (for IPv6) with the following exception:

        For IPv4-mapped IPv6-addresses the ``is_private`` value is determined by the
        semantics of the underlying IPv4 addresses and the following condition holds
        (see :attr:`IPv6Address.ipv4_mapped`)::

            address.is_global == address.ipv4_mapped.is_global

        ``is_global`` has value opposite to :attr:`is_private`, except for the ``100.64.0.0/10``
        IPv4 range where they are both ``False``.
        """
        ...
    @property
    def is_link_local(self) -> bool:
        """
        Test if the address is reserved for link-local.

        Returns:
            A boolean, True if the address is link-local per RFC 3927.
        """
        ...
    @property
    def is_loopback(self) -> bool:
        """
        Test if the address is a loopback address.

        Returns:
            A boolean, True if the address is a loopback per RFC 3330.
        """
        ...
    @property
    def is_multicast(self) -> bool:
        """
        Test if the address is reserved for multicast use.

        Returns:
            A boolean, True if the address is multicast.
            See RFC 3171 for details.
        """
        ...
    @property
    def is_private(self) -> bool:
        """
        ``True`` if the address is defined as not globally reachable by
        iana-ipv4-special-registry_ (for IPv4) or iana-ipv6-special-registry_
        (for IPv6) with the following exceptions:

        * ``is_private`` is ``False`` for ``100.64.0.0/10``
        * For IPv4-mapped IPv6-addresses the ``is_private`` value is determined by the
            semantics of the underlying IPv4 addresses and the following condition holds
            (see :attr:`IPv6Address.ipv4_mapped`)::

                address.is_private == address.ipv4_mapped.is_private

        ``is_private`` has value opposite to :attr:`is_global`, except for the ``100.64.0.0/10``
        IPv4 range where they are both ``False``.
        """
        ...
    @property
    def is_reserved(self) -> bool:
        """
        Test if the address is otherwise IETF reserved.

        Returns:
            A boolean, True if the address is within the
            reserved IPv4 Network range.
        """
        ...
    @property
    def is_unspecified(self) -> bool:
        """
        Test if the address is unspecified.

        Returns:
            A boolean, True if this is the unspecified address as defined in
            RFC 5735 3.
        """
        ...
    @property
    def packed(self) -> bytes:
        """The binary representation of this address."""
        ...
    if sys.version_info >= (3, 13):
        @property
        def ipv6_mapped(self) -> IPv6Address:
            """
            Return the IPv4-mapped IPv6 address.

            Returns:
                The IPv4-mapped IPv6 address per RFC 4291.
            """
            ...

class IPv4Network(_BaseV4, _BaseNetwork[IPv4Address]):
    def __init__(self, address: object, strict: bool = True) -> None: ...

class IPv4Interface(IPv4Address):
    netmask: IPv4Address
    network: IPv4Network
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    @property
    def hostmask(self) -> IPv4Address: ...
    @property
    def ip(self) -> IPv4Address: ...
    @property
    def with_hostmask(self) -> str: ...
    @property
    def with_netmask(self) -> str: ...
    @property
    def with_prefixlen(self) -> str: ...

class _BaseV6:
    __slots__ = ()
    if sys.version_info >= (3, 14):
        version: Final = 6
        max_prefixlen: Final = 128
    else:
        @property
        def version(self) -> Literal[6]: ...
        @property
        def max_prefixlen(self) -> Literal[128]: ...

class IPv6Address(_BaseV6, _BaseAddress):
    __slots__ = ("_ip", "_scope_id", "__weakref__")
    def __init__(self, address: object) -> None: ...
    @property
    def is_global(self) -> bool:
        """
        ``True`` if the address is defined as globally reachable by
        iana-ipv4-special-registry_ (for IPv4) or iana-ipv6-special-registry_
        (for IPv6) with the following exception:

        For IPv4-mapped IPv6-addresses the ``is_private`` value is determined by the
        semantics of the underlying IPv4 addresses and the following condition holds
        (see :attr:`IPv6Address.ipv4_mapped`)::

            address.is_global == address.ipv4_mapped.is_global

        ``is_global`` has value opposite to :attr:`is_private`, except for the ``100.64.0.0/10``
        IPv4 range where they are both ``False``.
        """
        ...
    @property
    def is_link_local(self) -> bool:
        """
        Test if the address is reserved for link-local.

        Returns:
            A boolean, True if the address is reserved per RFC 4291.
        """
        ...
    @property
    def is_loopback(self) -> bool:
        """
        Test if the address is a loopback address.

        Returns:
            A boolean, True if the address is a loopback address as defined in
            RFC 2373 2.5.3.
        """
        ...
    @property
    def is_multicast(self) -> bool:
        """
        Test if the address is reserved for multicast use.

        Returns:
            A boolean, True if the address is a multicast address.
            See RFC 2373 2.7 for details.
        """
        ...
    @property
    def is_private(self) -> bool:
        """
        ``True`` if the address is defined as not globally reachable by
        iana-ipv4-special-registry_ (for IPv4) or iana-ipv6-special-registry_
        (for IPv6) with the following exceptions:

        * ``is_private`` is ``False`` for ``100.64.0.0/10``
        * For IPv4-mapped IPv6-addresses the ``is_private`` value is determined by the
            semantics of the underlying IPv4 addresses and the following condition holds
            (see :attr:`IPv6Address.ipv4_mapped`)::

                address.is_private == address.ipv4_mapped.is_private

        ``is_private`` has value opposite to :attr:`is_global`, except for the ``100.64.0.0/10``
        IPv4 range where they are both ``False``.
        """
        ...
    @property
    def is_reserved(self) -> bool:
        """
        Test if the address is otherwise IETF reserved.

        Returns:
            A boolean, True if the address is within one of the
            reserved IPv6 Network ranges.
        """
        ...
    @property
    def is_unspecified(self) -> bool:
        """
        Test if the address is unspecified.

        Returns:
            A boolean, True if this is the unspecified address as defined in
            RFC 2373 2.5.2.
        """
        ...
    @property
    def packed(self) -> bytes:
        """The binary representation of this address."""
        ...
    @property
    def ipv4_mapped(self) -> IPv4Address | None:
        """
        Return the IPv4 mapped address.

        Returns:
            If the IPv6 address is a v4 mapped address, return the
            IPv4 mapped address. Return None otherwise.
        """
        ...
    @property
    def is_site_local(self) -> bool:
        """
        Test if the address is reserved for site-local.

        Note that the site-local address space has been deprecated by RFC 3879.
        Use is_private to test if this address is in the space of unique local
        addresses as defined by RFC 4193.

        Returns:
            A boolean, True if the address is reserved per RFC 3513 2.5.6.
        """
        ...
    @property
    def sixtofour(self) -> IPv4Address | None:
        """
        Return the IPv4 6to4 embedded address.

        Returns:
            The IPv4 6to4-embedded address if present or None if the
            address doesn't appear to contain a 6to4 embedded address.
        """
        ...
    @property
    def teredo(self) -> tuple[IPv4Address, IPv4Address] | None:
        """
        Tuple of embedded teredo IPs.

        Returns:
            Tuple of the (server, client) IPs or None if the address
            doesn't appear to be a teredo address (doesn't start with
            2001::/32)
        """
        ...
    @property
    def scope_id(self) -> str | None:
        """
        Identifier of a particular zone of the address's scope.

        See RFC 4007 for details.

        Returns:
            A string identifying the zone of the address if specified, else None.
        """
        ...
    def __hash__(self) -> int: ...
    def __eq__(self, other: object) -> bool: ...

class IPv6Network(_BaseV6, _BaseNetwork[IPv6Address]):
    def __init__(self, address: object, strict: bool = True) -> None: ...
    @property
    def is_site_local(self) -> bool:
        """
        Test if the address is reserved for site-local.

        Note that the site-local address space has been deprecated by RFC 3879.
        Use is_private to test if this address is in the space of unique local
        addresses as defined by RFC 4193.

        Returns:
            A boolean, True if the address is reserved per RFC 3513 2.5.6.
        """
        ...

class IPv6Interface(IPv6Address):
    netmask: IPv6Address
    network: IPv6Network
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    @property
    def hostmask(self) -> IPv6Address: ...
    @property
    def ip(self) -> IPv6Address: ...
    @property
    def with_hostmask(self) -> str: ...
    @property
    def with_netmask(self) -> str: ...
    @property
    def with_prefixlen(self) -> str: ...

def v4_int_to_packed(address: int) -> bytes: ...
def v6_int_to_packed(address: int) -> bytes: ...

# Third overload is technically incorrect, but convenient when first and last are return values of ip_address()
@overload
def summarize_address_range(first: IPv4Address, last: IPv4Address) -> Iterator[IPv4Network]: ...
@overload
def summarize_address_range(first: IPv6Address, last: IPv6Address) -> Iterator[IPv6Network]: ...
@overload
def summarize_address_range(
    first: IPv4Address | IPv6Address, last: IPv4Address | IPv6Address
) -> Iterator[IPv4Network] | Iterator[IPv6Network]: ...
def collapse_addresses(addresses: Iterable[_N]) -> Iterator[_N]: ...
@overload
def get_mixed_type_key(obj: _A) -> tuple[int, _A]: ...
@overload
def get_mixed_type_key(obj: IPv4Network) -> tuple[int, IPv4Address, IPv4Address]: ...
@overload
def get_mixed_type_key(obj: IPv6Network) -> tuple[int, IPv6Address, IPv6Address]: ...

class AddressValueError(ValueError): ...
class NetmaskValueError(ValueError): ...



# This sample tests error handling for PEP 696. TypeVars without default
# types cannot be after TypeVars with default types. This is the same as
# typeVarDefault3 except that it uses PEP 695 syntax.

from typing import TypeVar


# This should generate an error because T1 is after T2.
class ClassA[T2 = str, T1]: ...


# This should generate an error because T1 is after T2.
def funcA[T2 = str, T1](a: T2, b: T1) -> T1 | T2: ...


# This should generate an error because T1 is after T2.
type TA_A[T2 = str, T1] = dict[T2, T1]



# This sample tests the reporting of builtin types that
# will generate exceptions when subscripted in older
# versions of Python.

from queue import Queue
from collections import OrderedDict, deque
from asyncio import Future, Task


# These should generate errors for Python 3.8 and older.
def func1(
    a1: Queue[int],
    b1: OrderedDict[str, str],
    d1: list[int],
    e1: dict[str, int],
    f1: set[int],
    g1: deque[int],
    h1: frozenset[int],
    # These previously generated errors, but no longer do because of
    # changes in the typeshed stubs.
    c1: Future[int],
    i1: Task[None],
) -> None:
    pass


def func2(
    a1: "Queue[int]",
    b1: "OrderedDict[str, str]",
    c1: "Future[int]",
    d1: "list[int]",
    e1: "dict[str, int]",
    f1: "set[int]",
    g1: "deque[int]",
    h1: "frozenset[int]",
    i1: "Task[None]",
) -> None:
    pass


# These should generate errors because they are used
# in variable types, but they appear outside of a function.
class A:
    a1: Queue[int] = Queue()
    b1: OrderedDict[str, str] = OrderedDict()
    d1: list[int] = []
    e1: dict[str, int] = {}
    f1: set[int] = set()
    g1: deque[int] = deque()
    h1: frozenset[int] = frozenset()

    # These previously generated errors, but no longer do because of
    # changes in the typeshed stubs.
    c1: Future[int] = Future()
    i1: Task[None]


class B:
    a2: "Queue[int]" = Queue()
    b2: "OrderedDict[str, str]" = OrderedDict()
    c2: "Future[int]" = Future()
    d2: "list[int]" = []
    e2: "dict[str, int]" = {}
    f2: "set[int]" = set()
    g2: "deque[int]" = deque()
    h2: "frozenset[int]" = frozenset()
    i1: "Task[None]"


def func3():
    # These should not generate errors.
    a1: Queue[int] = Queue()
    b1: OrderedDict[str, str] = OrderedDict()
    c1: Future[int] = Future()
    d1: list[int] = []
    e1: dict[str, int] = {}
    f1: set[int] = set()
    g1: deque[int] = deque()
    h1: frozenset[int] = frozenset()
    i1: Task[None]



# This sample tests the constraint solver's handling of callables
# whose input parameters are contravariant.

from typing import TypeVar, Callable

T = TypeVar("T")
U = TypeVar("U")
V = TypeVar("V")


def compose2(f: Callable[[T], U], g: Callable[[U], V]) -> Callable[[T], V]:
    def composition(x: T) -> V:
        return g(f(x))

    return composition


def add_one(x: int) -> int:
    return x + 1


def make_str(x: int) -> str:
    return str(x)


add_two: Callable[[int], str] = compose2(add_one, make_str)



# This sample tests basic parsing of match statements as
# described in PEP 634.

from typing import Any

value_obj: Any = 4

class Foo:
    x: int


match (1, ):
    case a1, b1 if True:
        pass

    case (a2, b2):
        pass

    case [a3, b3]:
        pass

    case () | []:
        pass

    # This should generate an error because of a missing pattern.
    case :
        pass

    # This should generate an error because it is an irrefutable pattern
    # and is not at the end.
    case (a4):
        pass

    case (a5,):
        pass

    case [a6,]:
        pass

    case a7 as b7, c7 as d7 if True:
        pass

    case (a8, b8, ) as c8 if 1 == 3:
        pass

    case a9, *b8:
        pass

    # This should generate an error because multiple star
    # patterns in a sequence are not allowed.
    case *a10, *b10:
        pass

    # This should generate an error because star
    # patterns cannot be used with "as".
    case *a11 as b11, b12:
        pass

    case value_obj.a, value_obj.b:
        pass

    # This should generate an error because star
    # patterns can't be used with |.
    case (3 as b13) | (4 as b13) | *b13:
        pass

    case *a14, b14:
        pass

    case (a20, (b20,), [c20, *d20]) as e20:
        pass

    case 3 | -3:
        pass

    case 3.2 - 2.1j | -3.2 + 2.1j | 3j:
        pass

    # This should generate an error because the grammar
    # indicates that imaginary number must come second.
    case 2j + 4:
        pass

    # This should generate an error because the grammar
    # indicates that imaginary number must come second.
    case - 2j + 4:
        pass

    case "hi" """hi""" | r"hi" r"""hi""":
        pass

    # This should generate an error because f-strings are
    # not allowed.
    case "hi" f"""hi""":
        pass

    # This should generate an error.
    case {}:
        pass

    case {"a": 3, -3 + 4j: a30, value_obj.a: b30, **c30}:
        pass

    # This should generate an error because only one ** expression
    # can be used.
    case {"a": 3, **a31, "b": -3j, **b31}:
        pass

    # This should generate an error because ** cannot be used with
    # wildcard "_".
    case {"a": 3, **_, "b": -3}:
        pass

    case (3 as x) as y:
        pass

    case int():
        pass

    case Foo(1, a40, value_obj.b as b40, c40=3|-2 + 5j|"hi" as d40, y=[e40, f40] as g40,):
        pass
  
    # This should generate an error because positional arguments
    # cannot appear after keyword arguments.
    case Foo(1, a41, x=3, value_obj.b as b41, c41=3, y=[d41, e41] as f41):
        pass
  
    # This should generate three errors because irrefutable patterns
    # must appear only as the last entry in an or pattern.
    case (_ as x) | x:
        pass

    # This should generate an error because it's an irrefutable pattern
    # but is not the last case statement.
    case _:
        pass

    # This should generate an error because it's an irrefutable pattern
    # but is not the last case statement.
    case (x):
        pass

    case _ if value_obj:
        pass

    # This should generate an error because or patterns must target the
    # same names.
    case 3 | x:
        pass

    case _:
        pass



def func1():
    match = Foo()

    # This should be treated as an expression statement, not a match statement.
    match.x


def func2():
    match = [3]

    # This should be treated as an expression statement, not a match statement.
    match[0]

    match [0]:
        case _:
            pass

def func3():
    def match(a: int): ...

    # This should be treated as a call statement.
    match(0)

    match (0):
        case _:
            pass

def func4():
    match 1, 2, "3":
        case _:
            pass

def func5(match: Any):
    # This should be treated as a list, not a match statement.
    match[2:8, 2:8] = 0


class Point:
    def __init__(self, x: int, y: int) -> None:
        self.x = x
        self.y = y

def func6(subj: Any):
    match subj:
        # This should generate an error because a is used twice in the same pattern.
        case [a, *a]:
            pass
        
        case ([c, d] as f) | ([d, c] as f):
            pass
        
        # This should generate an error because h is used twice in the same pattern.
        case (g, 1 as h) as h:
            pass
        
        # This should generate an error because j is used twice in the same pattern.
        case Point(x=j, y=j):
            pass

def func7():
    match +1:
        case _:
            pass
        
    match -1:
        case _:
            pass

    match ~1:
        case _:
            pass




# This sample exercises the type analyzer's isinstance type narrowing logic.

from types import NoneType
from typing import (
    Any,
    Generic,
    Iterable,
    Iterator,
    Protocol,
    Sized,
    TypeVar,
    Union,
    runtime_checkable,
)

S = TypeVar("S")
T = TypeVar("T")


class UnrelatedClass:
    class_var1: int

    def __init__(self) -> None:
        self.property: None = None


class UnrelatedSubclass(UnrelatedClass):
    def __init__(self) -> None:
        self.property2: None = None


class SuperClass:
    class_var1: int

    def __init__(self) -> None:
        self.property: None = None


class MyClass1(SuperClass):
    class_var2: int

    def __init__(self) -> None:
        self.property2: None = None


class MyClass2(SuperClass):
    def __init__(self) -> None:
        self.property2: None = None


def f(instance: Union[SuperClass, UnrelatedClass], a: Any) -> None:
    if isinstance(instance, (MyClass1, UnrelatedSubclass, a)):
        print(instance.property)

        # This should generate two errors:
        # 'property2' is not a known member of 'SuperClass'
        # 'property2' is not a known member of 'UnrelatedClass'
        print(instance.property2)
    else:
        print(instance.property)

        # This should generate two errors:
        # 'property2' is not a known member of 'SuperClass'
        # 'property2' is not a known member of 'UnrelatedClass'
        print(instance.property2)


def g(cls: Union[type[SuperClass], type[UnrelatedClass]], a: Any) -> None:
    if issubclass(cls, (MyClass1, UnrelatedSubclass, a)):
        print(cls.class_var1)

        # This should generate two errors:
        # 'property2' is not a known member of 'SuperClass'
        # 'property2' is not a known member of 'UnrelatedClass'
        print(cls.class_var2)
    else:
        print(cls.class_var1)

        # This should generate two errors:
        # 'property2' is not a known member of 'SuperClass'
        # 'property2' is not a known member of 'UnrelatedClass'
        print(cls.class_var2)


# This code should analyze without any errors.
class TestClass1:
    def __init__(self) -> None:
        self.property = True


class TestClass2(TestClass1):
    pass


def func1(instance: TestClass2) -> None:
    # Although it's redundant for code to check for either
    # TestClass1 or TestClass2, the analyzer should be fine with it.
    if isinstance(instance, TestClass2):
        print(instance.property)

    if isinstance(instance, TestClass1):
        print(instance.property)


def func2(val: Union[int, None, str]) -> int | None:
    return None if isinstance((z := val), str) else z


# Test the special-case handling of isinstance with a
# "type" class.
def func3(ty: type[int]) -> type[int]:
    assert isinstance(ty, (type, str))
    return ty


def func4(ty: type[int]) -> type[int]:
    assert not isinstance(ty, str)
    return ty


def func5(ty: type[T]) -> type[T]:
    assert isinstance(ty, (type, str))
    return ty


def func6(ty: type[T]) -> type[T]:
    assert not isinstance(ty, str)
    return ty


def func6_2(ty: type[int] | int):
    if isinstance(ty, type):
        reveal_type(ty, expected_text="type[int]")
    else:
        reveal_type(ty, expected_text="int")


def func6_3(ty: type):
    if issubclass(ty, str):
        reveal_type(ty, expected_text="type[str]")
    else:
        reveal_type(ty, expected_text="type[Unknown]")


def func6_4(ty: Any):
    if issubclass(ty, str):
        reveal_type(ty, expected_text="type[str]")
    else:
        reveal_type(ty, expected_text="Any")


# Test the handling of protocol classes that support runtime checking.
def func7(a: Union[list[int], int]):
    if isinstance(a, Sized):
        reveal_type(a, expected_text="list[int]")
    else:
        reveal_type(a, expected_text="int")


# Test handling of member access expressions whose types change based
# on isinstance checks.


class Base1: ...


class Sub1_1(Base1):
    value: str


class Sub1_2(Base1):
    value: Base1


def handler(node: Base1) -> Any:
    if isinstance(node, Sub1_1):
        reveal_type(node.value, expected_text="str")
    elif isinstance(node, Sub1_2):
        reveal_type(node.value, expected_text="Base1")
        if isinstance(node.value, Sub1_1):
            reveal_type(node.value, expected_text="Sub1_1")


def func8a(a: int | list[int] | dict[str, int] | None):
    if isinstance(a, (str, (int, list, type(None)))):
        reveal_type(a, expected_text="int | list[int] | None")
    else:
        reveal_type(a, expected_text="dict[str, int]")


def func8b(a: int | list[int] | dict[str, int] | None):
    if isinstance(a, str | int | list | type(None)):
        reveal_type(a, expected_text="int | list[int] | None")
    else:
        reveal_type(a, expected_text="dict[str, int]")


TA1 = str | int | list | None


def func8c(a: int | list[int] | dict[str, int] | None):
    if isinstance(a, TA1):
        reveal_type(a, expected_text="int | list[int] | None")
    else:
        reveal_type(a, expected_text="dict[str, int]")


def func9(a: int | None):
    if not isinstance(a, NoneType):
        reveal_type(a, expected_text="int")
    else:
        reveal_type(a, expected_text="None")


class Base2(Generic[S, T]):
    pass


class Sub2(Base2[T, T]):
    pass


def func10(val: Sub2[str] | Base2[str, float]):
    if isinstance(val, Sub2):
        reveal_type(val, expected_text="Sub2[str] | Sub2[str | float]")


@runtime_checkable
class Proto1(Protocol):
    def f0(self, /) -> None: ...


@runtime_checkable
class Proto2(Proto1, Protocol):
    def f1(self, /) -> None: ...


def func11(x: Proto1):
    if isinstance(x, Proto2):
        reveal_type(x, expected_text="Proto2")
    else:
        reveal_type(x, expected_text="Proto1")


TA2 = list["TA3"] | dict[str, "TA3"]
TA3 = str | TA2


def func12(x: TA3) -> None:
    if isinstance(x, dict):
        reveal_type(x, expected_text="dict[str, str | list[TA3] | dict[str, TA3]]")
    else:
        reveal_type(x, expected_text="str | list[str | list[TA3] | dict[str, TA3]]")


def func13(x: object | type[object]) -> None:
    if isinstance(x, object):
        reveal_type(x, expected_text="object | type[object]")


def func14(x: Iterable[T]):
    if isinstance(x, Iterator):
        reveal_type(x, expected_text="Iterator[T@func14]")


class Base15(Generic[T]):
    value: T


class Child15(Base15[int]):
    value: int


def func15(x: Base15[T]):
    if isinstance(x, Child15):
        # This should generate an error. It's here just to ensure that
        # this code branch isn't marked unreachable.
        reveal_type(x, expected_text="Never")

        reveal_type(x, expected_text="Child15")
        reveal_type(x.value, expected_text="int")


def func16(x: Any):
    if isinstance(x, (int, int)):
        reveal_type(x, expected_text="int")


def func17(x: Any):
    if isinstance(x, (Union[int, int])):
        reveal_type(x, expected_text="int")



# This sample tests the reportUnusedImport diagnostic rule
# for multipart imports.

# This should result in an error if reportUnusedImport is enabled.
import package2.module1

# This should result in an error if reportUnusedImport is enabled.
import package2.module2

import package2.module3

import package2.module2 as dummy


def func1():
    print(package2.module3.a3)
    print(dummy)



# This sample tests the handling of the @override decorator as described
# in PEP 698.

from typing import Callable, Protocol
from typing_extensions import (  # pyright: ignore[reportMissingModuleSource]
    Any,
    overload,
    override,
)


class ClassA:
    def method1(self) -> None:
        pass


class ClassB:
    def method3(self) -> None:
        pass

    @overload
    def method5(self, x: int) -> int: ...

    @overload
    def method5(self, x: str) -> str: ...

    def method5(self, x: int | str) -> int | str: ...


class ClassC(ClassA, ClassB):
    @property
    @override
    # This should generate an error because prop_a doesn't
    # override anything in its base class.
    def prop_a(self) -> int:
        raise NotImplementedError

    @override
    def method1(self) -> None:
        pass

    def method2(self) -> None:
        pass

    @override
    def method3(self) -> None:
        pass

    @override
    # This should generate an error because method3 does not
    # override anything in a base class.
    def method4(self) -> None:
        pass

    @overload
    def method5(self, x: int) -> int: ...

    @overload
    def method5(self, x: str) -> str: ...

    @override
    def method5(self, x: int | str) -> int | str: ...

    @overload
    def method6(self, x: int) -> int: ...

    @overload
    def method6(self, x: str) -> str: ...

    @override
    # This should generate an error because method6 does not
    # override anything in a base class.
    def method6(self, x: int | str) -> int | str: ...


class ClassD(Any): ...


class ClassE(ClassD):
    @override
    def method1(self) -> None:
        pass


def evil_wrapper(func: Callable[..., Any], /):
    def wrapped(*args: Any, **kwargs: Any) -> Any:
        raise NotImplementedError

    return wrapped


class F:
    def method1(self):
        pass


class G(F):
    @override
    @evil_wrapper
    def method1(self):
        pass


class H(Protocol):
    pass


class I(H, Protocol):
    @override
    # This should generate an error because method1 isn't present
    # in the base.
    def method1(self):
        pass

    @overload
    @override
    # This should generate an error because method2 isn't present
    # in the base.
    def method2(self, x: int) -> int: ...
    @overload
    def method2(self, x: str) -> str: ...



# This sample tests the case where a constructor call is evaluated
# using bidirectional type inference in the case where the expected
# type is `Self`.

# pyright: strict

from __future__ import annotations

from typing import Any, Generic, Protocol, TypeVar
from typing_extensions import Self  # pyright: ignore[reportMissingModuleSource]

T_contra = TypeVar("T_contra", contravariant=True)
ThingT = TypeVar("ThingT", bound="Thing[Any]")


class Callback(Protocol[T_contra]):
    def __call__(self, message: T_contra, /) -> Any: ...


class Thing(Generic[T_contra]):
    def __init__(self, callback: Callback[T_contra]) -> None:
        self._callback: Callback[T_contra] = callback

    def copy(self) -> Self:
        return type(self)(self._callback)



# This sample tests that positional arg expressions are evaluated prior
# to keyword arg expressions even if they don't appear in that order
# within the arg list.


from typing import TypedDict


def func1(a: int | None = None, b: int | None = None, c: int | None = None) -> None:
    pass


func1((v1 := 1), b=v1 + 1)

# This should generate an error.
func1(b=(v2 := 1), *[v2 + 1])

func1(b=v3 + 1, *[(v3 := 1)])


class A(TypedDict):
    a: int


class B(TypedDict):
    b: int


class C(TypedDict):
    c: int


func1(a=(v4 := 1), **B(b=(v4 + 1)))

# This should generate an error.
func1(**A(a=(v5 + 1)), b=(v5 := 1))

func1(**A(a=(v5 := 1)), b=(v5 + 1))

func1(b=(v6 + 1), *[(v6 := 1)], **C(c=(v6 + 2)))


def func2(a: int, b: int):
    pass


func2(b=1, *(2,))



# This sample tests the case where a `type[T]` or `type[Self]` typevar is
# used as the base for a member access but is then used to call an
# instance method on the resulting class.

from contextlib import contextmanager


class A:
    @classmethod
    def method1(cls) -> None:
        cls.method2

    @contextmanager
    def method2(self):
        yield



# This sample tests the type checker's handling of
# synthesized __init__ and __new__ methods for
# dataclass classes and their subclasses.

from dataclasses import dataclass


@dataclass
class A:
    x: int


@dataclass(init=False)
class B(A):
    y: int

    def __init__(self, a: A, y: int):
        self.__dict__ = a.__dict__


a = A(3)
b = B(a, 5)


# This should generate an error because there is an extra parameter
a = A(3, 4)

# This should generate an error because there is one too few parameters
b = B(a)


A.__new__(A)
B.__new__(B)



# This sample tests the PEP 695 type parameter syntax extensions introduced
# in PEP 696 (default types for TypeVarLike).

from typing import Any, ParamSpec, TypeVar, Unpack
from typing_extensions import TypeVarTuple  # pyright: ignore[reportMissingModuleSource]

T1 = TypeVar("T1")
Ts1 = TypeVarTuple("Ts1")
P1 = ParamSpec("P1")


# This should generate an error because default must be a type expression.
class ClassT1[T = 3]: ...


class ClassT2[T: float = int]: ...


# This should generate an error because default must be a subtype of bound.
class ClassT3[T: int = float]: ...


class ClassT4[T: list[Any] = list[int]]: ...


class ClassT5[T: (bytes, str) = str]: ...


# This should generate an error because str | bytes isn't one of the constrained types.
class ClassT6[T: (bytes, str) = str | bytes]: ...


# This should generate an error because T1 is not a valid default.
class ClassT7[T = T1]: ...


# This should generate an error because Ts1 is not a valid default.
class ClassT8[T = Ts1]: ...


# This should generate an error because P1 is not a valid default.
class ClassT9[T = P1]: ...


class ClassTs1[*Ts = *tuple[int]]: ...


class ClassTs2[*Ts = Unpack[tuple[int]]]: ...


# This should generate an error because default must be unpacked tuple.
class ClassTs3[*Ts = tuple[int]]: ...


# This should generate an error because default must be unpacked tuple.
class ClassTs4[*Ts = int]: ...


# This should generate an error because default must be unpacked tuple.
class ClassTs5[*Ts = T1]: ...


# This should generate an error because default must be unpacked tuple.
class ClassTs6[*Ts = Ts1]: ...


# This should generate an error because default must be unpacked tuple.
class ClassTs7[*Ts = P1]: ...


class ClassTs8[*Ts = Unpack[tuple[int, ...]]]: ...


# This should generate an error because T1 isn't legal here.
class ClassTs9[*Ts = Unpack[tuple[T1, T1]]]: ...


# This should generate an error because ... isn't legal here.
class ClassTs10[*Ts = ...]: ...


class ClassP1[**P = [int]]: ...


class ClassP2[**P = ...]: ...


class ClassP3[**P = []]: ...


class ClassP4[**P = [int, str, None, int | None]]: ...


# This should generate an error because T1 isn't legal here.
class ClassP5[**P = [T1]]: ...


# This should generate an error because ParamSpec must be a list of types.
class ClassP6[**P = int]: ...


# This should generate an error because ParamSpec must be a list of types.
class ClassP7[**P = 3]: ...


# This should generate an error because ParamSpec must be a list of types.
class ClassP8[**P = [1, int]]: ...


# This should generate an error because it combines a traditional ParamSpec
# with a new-style (PEP 695) ParamSpec.
class ClassP9[**P = P1]: ...


# This should generate an error because ParamSpec must be a list of types.
class ClassP10[**P = Ts1]: ...



from __future__ import annotations

from pathlib import Path
from shutil import copytree, rmtree

from docify import run as docify  # pyright:ignore[reportMissingTypeStubs]


def main(*, overwrite: bool):
    """
    adds docstrings to the stubs located in `../stubdocs` for any modules that are compiled,
    because otherwise pyright would have no way to see the docstrings because there's no source
    code.

    note that it only generates stubs for modules and objects that exist on your current python
    version and OS.

    :param overwrite:
        whether to overwrite existing generated docstubs if they already exist. should be `True`
        when running locally to avoid running with a potentially outdated version of typeshed, but
        `False` in CI when generating them for all platforms because it needs to run multiple times
        on the previously generated output
    """
    stubs_path = Path("packages/pyright-internal/typeshed-fallback")
    stubs_with_docs_path = Path("docstubs")
    if overwrite:
        if stubs_with_docs_path.exists():
            rmtree(stubs_with_docs_path)
        copytree(stubs_path, stubs_with_docs_path, dirs_exist_ok=False)
    elif not stubs_with_docs_path.exists():
        copytree(stubs_path, stubs_with_docs_path, dirs_exist_ok=True)
    docify(
        input_dir=str(stubs_with_docs_path / "stdlib"),
        if_needed=True,
        in_place=True,
        workers=0,  # 0 means automatically determine the number of workers based on the cpu
    )


if __name__ == "__main__":
    main(overwrite=False)



# This sample tests the custom __call__ method on the EnumMeta class.

from enum import Enum


class Foo(Enum):
    A = 1
    B = 2


Foo(1)

# This would have previously generated an error prior to Python 3.12,
# but it now does not because of an additional overload on the EnumMeta
# __call__ method.
Foo(1, 2, 3, 4)



# This sample tests the type analyzer's handling of TypedDict classes.

from typing import Any, TypeVar, TypedDict

not_total = False


# This should generate an error because
# the value of the total argument must
# be a literal "True" or "False".
class TD1(TypedDict, total=not_total):
    pass


class TD2(TypedDict, total=False):
    """This is a test"""

    a: int

    # This should generate an error because "b"
    # is redeclared below with a different type.
    b: str

    b: float

    c: "dict[Any, Any]"

    # This should generate an error because
    # assignments are not allowed.
    d: float = 3.0

    # This should generate an error because
    # methods are not allowed.
    def foo(self):
        pass


class TD3(TypedDict, total=True):
    a: int
    b: float
    c: str


class TD4(TypedDict):
    d: str


class TD5(TD3, total=False):
    e: str

    # This should generate an error because
    # methods are not allowed.
    def foo(self):
        pass


class NotATD:
    pass


# This should generate an error because non-TypeDict
# base classes shouldn't be allowed for TD classes.
class TD6(TD3, NotATD):
    pass


# This should generate an error because non-TypeDict
# base classes shouldn't be allowed for TD classes.
class TD7(NotATD, TypedDict):
    pass


# This should generate an error because TypedDict can't
# be used in a type annotation.
v1: TypedDict | int

# This should generate an error because TypedDict can't
# be used in a TypeVar bound.
T = TypeVar("T", bound=TypedDict | int)


# This should generate an error because TypedDict doesn't support
# a metaclass parameter.
class TD8(TypedDict, metaclass=type):
    name: str


# This should generate an error because TypedDict doesn't support
# other __init_subclass__ parameters.
class TD9(TypedDict, other=True):
    name: str



# This sample tests the case where two recursive type aliases
# with different definitions overlap.

from typing import Mapping, Optional, Sequence, Union

JsonArr1 = Sequence[Optional["JsonVal1"]]
JsonObj1 = Mapping[str, Optional["JsonVal1"]]
JsonVal1 = Union[bool, float, int, str, "JsonArr1", "JsonObj1"]

JsonArr2 = Sequence[Optional["JsonVal2"]]
JsonObj2 = Mapping[str, Optional["JsonVal2"]]
JsonVal2 = Union[bool, float, int, str, "JsonArr2", "JsonObj2"]


def func1(v: JsonVal1):
    x: JsonVal2 = v

    return x


def func2(v: Optional[JsonVal1]):
    # This should generate an error.
    x: JsonVal2 = v

    return x


def func3(v: Optional[JsonVal1]):
    # This should generate an error.
    x: Optional[JsonVal2] = v

    return x



# This sample tests the type checker's validation of class variables
# whose name conflict with a __slots__ entry.

from dataclasses import dataclass, field


class NoSlots1:
    pass


class Slots1(NoSlots1):
    __slots__ = "aaa", "bbb", "ccc"

    # This should generate an error
    aaa = 3

    # This should generate an error
    bbb: int = 3

    # This should generate an error
    (ccc, ddd) = 3, 4

    eee = 5


class Slots2(Slots1):
    __slots__ = ()

    aaa = 4


@dataclass
class Slots3:
    __slots__ = ("values",)

    # This should not generate an error because class variables
    # in a dataclass are replaced by instance variables.
    values: list[int] = field(default_factory=list)



import dataclasses
import functools
from dataclasses import dataclass
from typing import final


@dataclass()
class A: ...

@dataclasses.dataclass()
class B:
    @final
    def method(self): ...
    @staticmethod
    def static(): ...

@functools.cache
def cached(): ...

B.static()



# This sample tests type narrowing of instance variables in the presence
# of a double nested loop.


def func1(x: str | None):
    assert x is not None

    for i in range(10):
        for j in range(10):
            x = x + ""



# This sample tests the reportIncompatibleMethodOverride
# diagnostic check.


from typing import Any, Generic, ParamSpec, Self, TypeVar


class Base1:
    def f1(self, *, kwarg0: int) -> None: ...

    def f2(self, *, kwarg0: int) -> None: ...

    def f3(self, *, kwarg0: int) -> None: ...

    def f4(self, *, kwarg0: int) -> None: ...

    def g1(self, a: int, /, b: str, *, kwarg0: int) -> None: ...

    def g2(self, a: int, /, b: str, *, kwarg0: int) -> None: ...

    def g3(self, a: int, /, b: str, *, kwarg0: int) -> None: ...

    def g4(self, a: int, /, b: str, *, kwarg0: int) -> None: ...

    def g5(self, a: int, /, b: str, *, kwarg0: int) -> None: ...

    def g6(self, a: int, /, b: str, *, kwarg0: int) -> None: ...

    def h1(self, a: int, *args: int) -> None: ...


class Derived1(Base1):
    def f1(self, arg0: int = 0, *, kwarg0: int, kwarg1: int = 0) -> None: ...

    # This should generate an error because of a positional parameter mismatch.
    def f2(self, arg0: int, *, kwarg0: int, kwarg1: int = 0) -> None: ...

    # This should generate an error because of a missing kwarg1.
    def f3(self, arg0: int = 0, *, kwarg0: int, kwarg1: int) -> None: ...

    # This should generate an error because kwarg0 is the wrong type.
    def f4(self, arg0: int = 0, *kwarg0: str) -> None: ...

    def g1(self, xxx: int, /, b: str, *, kwarg0: int) -> None: ...

    def g2(self, __a: int, b: str, *, kwarg0: int) -> None: ...

    # This should generate an error because of a name mismatch between b and c.
    def g3(self, __a: int, c: str, *, kwarg0: int) -> None: ...

    # This should generate an error because of a type mismatch for b.
    def g4(self, __a: int, b: int, *, kwarg0: int) -> None: ...

    def g5(self, __a: int, b: str = "hi", *, kwarg0: int) -> None: ...

    def g6(self, __a: int, b: str, c: str = "hi", *, kwarg0: int) -> None: ...


P = ParamSpec("P")
R = TypeVar("R")


class Base2(Generic[P, R]):
    def method1(self, *args: P.args, **kwargs: P.kwargs) -> R: ...

    def method2(self, *args: P.args, **kwargs: P.kwargs) -> R: ...


class Derived2(Base2[P, R]):
    def method1(self, *args: P.args, **kwargs: P.kwargs) -> R: ...

    def method2(self, *args: Any, **kwargs: Any) -> R: ...


T = TypeVar("T")


class Base3:
    def method1(self, x: Self) -> Self: ...

    def method2(self, x: Self) -> Self: ...

    def method3(self, x: Self) -> Self: ...


class Derived3(Generic[T], Base3):
    # This should generate an error.
    def method1(self, x: "Derived3[T]") -> "Derived3[T]": ...

    # This should generate an error.
    def method2(self, x: "Derived3[int]") -> "Derived3[int]": ...

    # This should generate an error.
    def method3(self, x: Self) -> Self: ...



# This sample tests that ParamSpecs support parameters with default values.

from typing import Callable, ParamSpec, TypeVar

P = ParamSpec("P")
R = TypeVar("R")


def func1(fn: Callable[P, R]) -> Callable[P, R]: ...


def func2(a: str, b: str = "") -> str: ...


func1(func2)("")



# This sample tests the case where a NamedTuple object is referenced
# through a `self` parameter.

from typing import NamedTuple


class Fruit(NamedTuple):
    name: str
    cost: float

    def new_cost(self, new_cost: float):
        my_name, my_cost = self
        reveal_type(my_name, expected_text="str")
        reveal_type(my_cost, expected_text="float")
        return Fruit(my_name, new_cost)



# This sample tests a limiter that prevents infinite recursion
# in the tuple inference logic.


def func1(val: int):
    t = None
    while True:
        t = (val or t, val)
        val += 1
        if val > 1000:
            break
    return t



# This sample tests bidirectional type inference and constraint solving.

m = int(1)
n = float(1.1)
p = "hello"

a = dict(x=m, y=m)
a1: int = a["x"]

b = dict(x=n, y=n)
reveal_type(b, expected_text="dict[str, float]")

# This should generate an error.
b1: int = b["x"]
b2: float = b["x"]

c = dict(x=m, y=n)
reveal_type(c, expected_text="dict[str, float]")

# This should generate an error.
c1: int = c["x"]
c2: float = c["x"]

d = dict(x=p, y=p)
reveal_type(d, expected_text="dict[str, str]")

# This should generate an error.
d1: float = d["x"]
d2: str = d["x"]

e = dict(x=n, y=p)
reveal_type(e, expected_text="dict[str, float | str]")

# This should generate an error.
e1: str = e["x"]
# This should generate an error.
e2: float = e["x"]
e3: float | str = e["x"]



# This sample tests the functools.partial support.

from functools import partial
from typing import Callable, Protocol, Self, TypeVar

_T1 = TypeVar("_T1")
_T2 = TypeVar("_T2", covariant=True)


def func1():
    """func1"""
    return 0


p1_0 = partial(func1)

reveal_type(p1_0(), expected_text="Literal[0]")

# This should generate an error.
p1_0("")

# This should generate an error.
p1_1 = partial(func1, "", "")


def func2(name: str, number: int) -> None:
    """func2"""
    pass


p2_0 = partial(func2)

reveal_type(p2_0("", 3), expected_text="None")

# This should generate an error.
p2_0()

# This should generate an error.
p2_0("")

# This should generate an error.
p2_0("", 3, 3)

# This should generate an error.
p2_0("", 3, 3)

p2_0("", number=3)

# This should generate an error.
p2_0("", 3, number=3)

p2_1 = partial(func2, "")

# This should generate an error.
p2_1()

p2_1(3)
p2_1(number=3)

# This should generate an error.
p2_1(3, number=3)

p2_2 = partial(func2, number=3)
p2_2("")

p2_3 = partial(func2, number=3, name="")
p2_3()


def func3(name: str, /, number: int):
    return 0


p3_0 = partial(func3)

reveal_type(p3_0("", 3), expected_text="Literal[0]")

# This should generate an error.
p3_0(name="", number=3)

p3_1 = partial(func3, "")

p3_1(3)
p3_1(number=3)


def func4(name: str, *, number: int):
    return 0


p4_0 = partial(func4)

# This should generate an error.
p4_0("", 3)

p4_0("", number=3)


def func5(name: _T1, number: _T1) -> _T1:
    return name


p5_0 = partial(func5)
reveal_type(p5_0(3, 3), expected_text="int")
reveal_type(p5_0("3", "3"), expected_text="str")


p5_1 = partial(func5, 2)

p5_1(3)

# This should generate an error.
p5_1("3")


def func6(a: int, name: _T1, number: _T1) -> _T1:
    return name


p6_0 = partial(func6, 3, 4)

reveal_type(p6_0(3), expected_text="int")


def func7(a: int, name: float, *args: str):
    return 0


p7_0 = partial(func7, 3, 3, "", "", "")
p7_0("", "")

# This should generate an error.
p7_0(3)

p7_1 = partial(func7)
p7_1(3, 0)
p7_1(3, 0, "", "")

# This should generate an error.
p7_1(3, 0, foo=3)


def func8(a: int, name: str, **kwargs: int):
    return 0


p8_0 = partial(func8, 3, "")
p8_0()
p8_0(foo=3)

# This should generate an error.
p8_0(foo="")

p8_1 = partial(func8)
p8_1(3, "")

# This should generate an error.
p8_1(3)


# This should generate an error.
p8_1(3, "", 5)

p8_1(3, "", foo=4, bar=5)


class Partial(Protocol[_T2]):
    def __new__(cls, __func: Callable[..., _T2]) -> Self: ...


def func9() -> int: ...


# This should generate an error.
x: Partial[str] = partial(func9)



# This sample tests the case where a lambda is passed to a generic
# Callable with two different type variables.

from typing import Callable, Generic, TypeVar

T = TypeVar("T")
R = TypeVar("R")


class A(Generic[T, R]):
    def __init__(self, x: Callable[[T], R], y: T): ...


class B(Generic[R]):
    def __init__(self, x: Callable[[T], R], y: T): ...


reveal_type(A(lambda x: x, 123), expected_text="A[int, int]")
reveal_type(B(lambda x: x, 123), expected_text="B[int]")



from re import Pattern
from sre_constants import *
from sre_constants import _NamedIntConstant
from sre_parse import SubPattern
from typing import Any, Final

MAXCODE: Final[int]

def dis(code: list[_NamedIntConstant]) -> None: ...
def isstring(obj: Any) -> bool: ...
def compile(p: str | bytes | SubPattern, flags: int = 0) -> Pattern[Any]: ...



# This sample tests the case where an enum class is used as a bound
# for a TypeVar and instantiated.

# pyright: strict

from enum import Enum
from typing import TypeVar

TEnum = TypeVar("TEnum", bound=Enum)


def func1(enum_cls: type[TEnum], enum_value: object) -> TEnum:
    enum_member = enum_cls(enum_value)
    reveal_type(enum_member, expected_text="TEnum@func1")
    return enum_member



# This sample tests the case where a variable type declaration is found
# within a loop and the variable is used within a conditional expression
# within the same loop.

from enum import Enum


class MyEnum(Enum):
    A = 0


def func1(vals: list[MyEnum]):
    for val1 in vals:
        val2: MyEnum = val1
        if val2 == MyEnum.A:
            pass



# This sample tests bidirectional type inference for cases that involve
# constructors and dict inference.

from typing import TypeVar

T = TypeVar("T")


def func1(some_dict: dict[str, set[T] | frozenset[T]]) -> list[T]:
    return []


v1 = func1({"foo": set({1, 2, 3})})
reveal_type(v1, expected_text="list[int]")

v2 = func1({"foo": frozenset({1, 2, 3})})
reveal_type(v2, expected_text="list[int]")



# This sample tests narrowing of subject subexpressions in match statements.

from typing import Literal, TypedDict


class TD1(TypedDict):
    name: Literal["a"]
    extra_value: int


class TD2(TypedDict):
    name: Literal["b"]
    other_extra_value: int


class TD3(TypedDict):
    name: Literal["c"]
    extra_value: int


def func1(item: TD1 | TD2):
    match item["name"]:
        case "d":
            reveal_type(item, expected_text="Never")
        case "a":
            reveal_type(item, expected_text="TD1")
        case "b":
            reveal_type(item, expected_text="TD2")


def func2(item: TD1 | TD2 | TD3):
    match item["name"]:
        case "a" | "c":
            reveal_type(item, expected_text="TD1 | TD3")
        case _:
            reveal_type(item, expected_text="TD2")


T1 = tuple[Literal[0], int]
T2 = tuple[Literal[1], str]


def func3(item: T1 | T2):
    match item[0]:
        case 0:
            reveal_type(item, expected_text="tuple[Literal[0], int]")
        case 1:
            reveal_type(item, expected_text="tuple[Literal[1], str]")


def func4(a: object, b: int) -> None:
    match a, b:
        case (complex(), 3):
            reveal_type(a, expected_text="complex")
            reveal_type(b, expected_text="Literal[3]")


Token = (
    str
    | tuple[Literal["define"], str, str]
    | tuple[Literal["include"], str]
    | tuple[Literal["use"], str, int, int]
)


def func5(token: Token):
    match token:
        case str(x):
            reveal_type(token, expected_text="str")
        case "define", _, _:
            reveal_type(token, expected_text="tuple[Literal['define'], str, str]")
        case "include", _:
            reveal_type(token, expected_text="tuple[Literal['include'], str]")
        case "use", _, _, _:
            reveal_type(token, expected_text="tuple[Literal['use'], str, int, int]")
        case _:
            reveal_type(token, expected_text="Never")


def func6(a: int | str, b: int | str) -> None:
    match a, b:
        case (_, _):
            reveal_type(a, expected_text="int | str")
            reveal_type(b, expected_text="int | str")
        case (x, y):
            reveal_type(x, expected_text="Never")
            reveal_type(y, expected_text="Never")
            reveal_type(a, expected_text="Never")
            reveal_type(b, expected_text="Never")


def func7(a: str | None, b: str | None) -> None:
    match (a, b):
        case (_, None):
            return
        case (None, _):
            return
    reveal_type(a, expected_text="str")
    reveal_type(b, expected_text="str")



# This sample tests the case where a lambda's expected type is a callable
# that accepts another generic callable as a parameter.

from typing import Callable, Generic, TypeVar

T = TypeVar("T")


def func1(callable: Callable[[Callable[[T], T]], T]) -> T:
    return callable(lambda x: x)


v1 = func1(lambda a: a(0))
reveal_type(v1, expected_text="int")

v2 = func1(lambda a: a(""))
reveal_type(v2, expected_text="str")


def test1(untyped):
    v1 = func1(lambda a: a(untyped))
    reveal_type(v1, expected_text="Unknown")


class A(Generic[T]):
    def __init__(self, value: T) -> None:
        self.value = value


def func2(callable: Callable[[type[A[T]]], A[T]]) -> T:
    return callable(A).value


v3 = func2(lambda A: A(0))
reveal_type(v3, expected_text="int")

v4 = func2(lambda A: A(""))
reveal_type(v4, expected_text="str")


def test2(untyped):
    v1 = func2(lambda A: A(untyped))
    reveal_type(v1, expected_text="Unknown")



# This sample tests the syntax handling for Python 3.11 exception groups
# as described in PEP 654.


def func1():

    try:
        pass

    # This should generate an error if using Python 3.10 or earlier.
    except* ValueError as e:
        reveal_type(e, expected_text="BaseExceptionGroup[ValueError]")
        pass

    # This should generate an error if using Python 3.10 or earlier.
    except*:
        pass



# This sample tests the case where a class decorator needs to evaluate
# the type of __init__ prior to the class being fully evaluated.

from typing import Any, Callable, Generic, TypeVar

T = TypeVar("T")
FuncType = Callable[..., Any]
FT = TypeVar("FT", bound=FuncType)


def decorate() -> Callable[[FT], FT]: ...


@decorate()
class ValueDecorated(Generic[T]):
    def __init__(self, value: T) -> None:
        self._value: T = value

    def __call__(self) -> T:
        return self._value



# This sample tests the case where a callback protocol uses
# an overloaded __call__ method.

from typing import Any, Protocol, overload


class P1(Protocol):
    @overload
    def __call__(self, x: int) -> int: ...

    @overload
    def __call__(self, x: str) -> str: ...

    def __call__(self, x: Any) -> Any: ...


def func0(x: Any) -> Any:
    return x


def func1(x: int) -> Any:
    return x


a0: P1 = func0

# This should generate an error
a1: P1 = func1


@overload
def of1(x: int) -> int: ...


@overload
def of1(x: str) -> str: ...


def of1(x: Any) -> Any:
    return x


@overload
def of2(x: int) -> complex: ...


@overload
def of2(x: str) -> str: ...


def of2(x: Any) -> Any:
    return x


b0: P1 = of1

# This should generate an error
b1: P1 = of2


class P2(Protocol):
    def __call__(self, *args: int) -> Any: ...


a: P2 = lambda *args: map(lambda arg: arg + 0, args)



# This sample tests the handling of variadic type variables used
# in generic type aliases and with suffixes.

from typing import Callable, Generic, TypeVar
from typing_extensions import (  # pyright: ignore[reportMissingModuleSource]
    TypeVarTuple,
    Unpack,
)


Ts = TypeVarTuple("Ts")
T = TypeVar("T", covariant=True)


class Call(Generic[Unpack[Ts]]):
    def __init__(self, *args: Unpack[Ts]) -> None:
        self.args = args


class Return(Generic[T]):
    def __init__(self, /, result: T) -> None:
        self.result = result


TailRec = Call[Unpack[Ts]] | Return[T]


def tail_rec(
    fn: Callable[[Unpack[Ts]], TailRec[Unpack[Ts], T]],
) -> Callable[[Unpack[Ts]], T]: ...


@tail_rec
def factorial(n: int, acc: int) -> TailRec[int, int, int]:
    if n <= 0:
        return Return(acc)
    return Call(n - 1, acc * n)


reveal_type(factorial, expected_text="(int, int) -> int")


Alias10 = tuple[T, *Ts]
Alias11 = tuple[*Ts]
Alias12 = tuple[T, *Ts, T]


def func5(a10: Alias10, a11: Alias11, a12: Alias12):
    reveal_type(a10, expected_text="tuple[Unknown, *tuple[Unknown, ...]]")
    reveal_type(a11, expected_text="tuple[Unknown, ...]")
    reveal_type(a12, expected_text="tuple[Unknown, *tuple[Unknown, ...], Unknown]")



# This sample tests various type checking operations relating to
# async generator functions where the return type is declared.

from typing import AsyncIterable, AsyncIterator


async def g1_explicit() -> AsyncIterator[int]:
    yield 1
    yield 2


async def g2_explicit():
    async for v in g1_explicit():
        yield v


async def g3_explicit() -> AsyncIterable[int]:
    yield 1
    yield 2


async def g4_explicit():
    async for v in g3_explicit():
        yield v



# This sample tests constraint solving for constrained type vars.

import pathlib
import shutil
from typing import AnyStr, Type, TypeVar, Union


class Foo:
    pass


class Bar(Foo):
    pass


T1 = TypeVar("T1", Foo, str)
T2 = TypeVar("T2", bound=Foo)


def test1(x: T1) -> T1:
    return x


def test2(x: T2) -> T2:
    return x


# This should generate an error because test1(Bar())
# should evaluate to type Foo, not Bar.
aa1: Bar = test1(Bar())

aa2: Foo = test1(Bar())

bb1: Bar = test2(Bar())

bb2: Foo = test2(Bar())


# The call to rmtree should not generate any errors.
data_dir = pathlib.Path("/tmp")
archive_path = data_dir / "hello"
shutil.rmtree(archive_path)


def func1(a: AnyStr, b: AnyStr) -> None: ...


def func2(a: Union[str, bytes], b: Union[str, bytes]):
    # This should generate two errors
    func1(a, b)


class A: ...


class B: ...


class C: ...


class D: ...


T3 = TypeVar("T3", A, B, Union[C, D])


def do_something(value: T3) -> T3: ...


def func10(value: Union[C, D]):
    value1 = do_something(value)


def func11(value: D):
    value1 = do_something(value)


def func12(value: Union[A, B]):
    # This should generate an error because A and B
    # map to different constraints.
    value1 = do_something(value)


def func13(value: Union[A, D]):
    # This should generate an error because A and D
    # map to different constraints.
    value1 = do_something(value)


T4 = TypeVar("T4", A, B, Union[C, D])


def func14(cls: Type[T4]) -> T4:
    instance1 = cls()
    reveal_type(instance1, expected_text="T4@func14")  # Unknown
    return instance1


def func15(cls: Union[Type[Union[A, B]], Type[Union[C, D]]]) -> Union[A, B, C, D]:
    instance2 = cls()
    reveal_type(instance2, expected_text="A | B | C | D")
    return instance2



# This sample tests the reporting of a function or class decorated with
# @type_check_only when used in a value expression.

from __future__ import annotations

from typing import TYPE_CHECKING, type_check_only

if TYPE_CHECKING:
    from typing import _TypedDict

a1: function
a2: _TypedDict

# This should generate an error.
v1 = function

# This should generate an error, but it doesn't because
# of a typeshed issue.
v2 = isinstance(1, ellipsis)

# This should generate an error.
v3 = _TypedDict


if TYPE_CHECKING:

    class ClassA:
        @type_check_only
        def method1(self):
            pass

    @type_check_only
    def func1() -> None: ...


# This should generate an error.
ClassA().method1()

# This should generate an error.
func1()



# This sample tests the cases where a mixin class
# overrides an abstract method, making it no longer abstract.

import abc


class InterfaceA(abc.ABC):
    @abc.abstractmethod
    def a(self) -> None:
        print("InterfaceA.a")


class MixinA(InterfaceA):
    def a(self) -> None:
        print("MixinA.a")


class InterfaceAB(InterfaceA):
    @abc.abstractmethod
    def b(self) -> None:
        print("InterfaceAB.b")


class ClassAB(InterfaceAB, MixinA):
    def b(self) -> None:
        print("ClassAB.b")


ab = ClassAB()
ab.a()



# This sample handles the case where a property setter contains
# a function-scoped TypeVar.

from typing import Hashable, TypeVar, Sequence

HashableT = TypeVar("HashableT", bound=Hashable)


class ClassA:
    def __init__(self):
        self._something = []

    @property
    def something(self) -> Sequence[Hashable]:
        return self._something

    @something.setter
    def something(self, thing: list[HashableT]):
        self._something = thing


f = ClassA()
f.something = ["a", "b", "c"]



# This sample covers the case where a function accepts a generic callable
# as a parameter along with another parameter that uses the same type variable
# and a caller provides an overloaded function as an argument.

from typing import Any, Callable, TypeVar, overload

T = TypeVar("T")


@overload
def func1(real: float): ...


@overload
def func1(real: str): ...


def func1(real: float | str) -> None: ...


def func2(f: Callable[[T], Any], p: T):
    return f(p)


func2(func1, 4)
func2(func1, "4")

# This should generate an error because a "bytes" argument
# doesn't match any of the overloads.
func2(func1, b"")


map(complex, ["3j", "4"])

# This should generate two errors because a "bytes" argument
# doesn't match any of the overloads in the "complex" constructor.
map(complex, [b"3j"])



# This sample tests the case where a class is parameterized by a ParamSpec
# which is inferred by a call to the constructor, and the passed value
# is a generic function whose types are informed by additional parameters
# also passed to the constructor.

from typing import Callable, Generic, ParamSpec, TypeVar

P = ParamSpec("P")
T = TypeVar("T")


class ABase: ...


class A(ABase): ...


TA = TypeVar("TA", bound=ABase)


class B(Generic[P, T]):
    def __init__(
        self, _type: Callable[P, T], *args: P.args, **kwargs: P.kwargs
    ) -> None: ...


def func1(t: type[TA]) -> TA: ...


b = B(func1, A)
reveal_type(b, expected_text="B[(t: type[A]), A]")


class C(Generic[TA]):
    def __init__(self, _type: type[TA]) -> None: ...


c = B(C, A)
reveal_type(c, expected_text="B[(_type: type[A]), C[A]]")



# This sample tests assignment of literals to declared
# types that use literals in their type arguments.

from typing import Dict, List, Literal, Set, TypeVar

Number = Literal["One", "Two"]

# This should generate an error because 'Three' is not
# allowed in the Number type.
numbers_mapping: Dict[Number, int] = {"One": 1, "Two": 2, "Three": 3}

# This should generate an error because 'Three' is not
# allowed in the Number type.
a: List[Number] = ["Three"]

# This should generate an error because 'Three' is not
# allowed in the Number type.
b: Set[Number] = {"One", "Three"}


LetterGrade = Literal["A", "B", "C", "D", "F"]

_T = TypeVar("_T")


def func1(x: _T) -> _T: ...


grade: LetterGrade = func1("A")



# This sample tests the support for functools.total_ordering.

from functools import total_ordering


@total_ordering
class ClassA:
    val1: int

    def __gt__(self, other: object) -> bool: ...


a = ClassA()
b = ClassA()
v1 = a < b
v2 = a <= b
v3 = a > b
v4 = a >= b
v5 = a == b
v6 = a != b


# This should generate an error because it doesn't declare
# any of the required ordering functions.
@total_ordering
class ClassB:
    val1: int


@total_ordering
class ClassC:
    def __eq__(self, other: object) -> bool:
        return False

    def __lt__(self, other: "ClassC") -> bool:
        return False


reveal_type(ClassC() < ClassC(), expected_text="bool")
reveal_type(ClassC() <= ClassC(), expected_text="bool")
reveal_type(ClassC() == ClassC(), expected_text="bool")
reveal_type(ClassC() > ClassC(), expected_text="bool")
reveal_type(ClassC() >= ClassC(), expected_text="bool")

_ = ClassC() == 1
_ = ClassC() != 1

# The following four lines should each produce an error.
_ = ClassC() < 1
_ = ClassC() <= 1
_ = ClassC() > 1
_ = ClassC() >= 1


@total_ordering
class ClassD:
    def __init__(self) -> None:
        self.value: int = 0

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, ClassD):
            return NotImplemented

        reveal_type(other, expected_text="ClassD")

        return self.value == other.value

    def __le__(self, other: object) -> bool:
        if not isinstance(other, ClassD):
            return NotImplemented

        return self.value <= other.value



# This sample tests that default parameter values can be assigned
# to types that are generic.

from typing import Generic, List, Type, TypeVar


class ClassA:
    pass


T_A = TypeVar("T_A", bound=ClassA)
T = TypeVar("T")


class ClassB(Generic[T_A, T]):
    def __init__(
        self,
        p1: Type[T_A] = ClassA,
        p2: List[T] = [],
        # This should generate an error.
        p3: List[T_A] = [2],
        p4: List[T] = [2],
    ):
        pass



# This sample tests isinstance and issubclass type narrowing
# based on cls and self parameters.


class ClassA:
    @classmethod
    def bar(cls, other: type):
        if issubclass(other, cls):
            reveal_type(other, expected_text="type[Self@ClassA]")

        if issubclass(other, (int, cls)):
            reveal_type(other, expected_text="type[Self@ClassA] | type[int]")

    def baz(self, other: object):
        if isinstance(other, type(self)):
            reveal_type(other, expected_text="Self@ClassA")

        if isinstance(other, (int, type(self))):
            reveal_type(other, expected_text="Self@ClassA | int")



# This sample tests a case where a method-scoped ParamSpec is used within one
# of several overloads but not in others.

from typing import Callable, Concatenate, overload, Any
from typing_extensions import (  # pyright: ignore[reportMissingModuleSource]
    ParamSpec,
    Self,
)

P = ParamSpec("P")


class A:
    @overload
    def method1(
        self,
        cb: Callable[Concatenate[Self, P], None],
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> None: ...

    @overload
    def method1(
        self, cb: tuple[Callable[..., None], str], *args: Any, **kwargs: Any
    ) -> None: ...

    def method1(self, cb, *args, **kwargs) -> None:
        if isinstance(cb, tuple):
            cb[0](self, *args, **kwargs)
        else:
            cb(self, *args, **kwargs)


def func1(fo: A, x: int) -> None: ...


def func2(fo: A, x: int, /, y: str) -> None: ...


def func3(fo: A, x: int, /, y: str, *, z: tuple[int, int]) -> None: ...


a = A()

a.method1(func1, 1)
a.method1(func2, 3, "f1")
a.method1(func3, 6, "f2", z=(0, 1))

a.method1((func1, "f1"), 1)
a.method1((func2, "f2"), 2, "a")
a.method1((func3, "f3"), 3, "b", z=(0, 1))



# This sample tests a complicated combination of ParamSpec usage.

# pyright: strict

from typing import Any, Callable, Concatenate, ParamSpec, Protocol, TypeVar


_Fn = TypeVar("_Fn", bound=Callable[..., Any])
_Ret = TypeVar("_Ret")
_Args = ParamSpec("_Args")
_Self = TypeVar("_Self", bound="_GenerativeType")


def decorator(
    target: Callable[Concatenate[_Fn, _Args], _Ret],
) -> Callable[[_Fn], Callable[_Args, _Ret]]: ...


class _GenerativeType(Protocol):
    def _generate(self: "_Self") -> "_Self": ...


def generative(
    fn: Callable[Concatenate[_Self, _Args], None],
) -> Callable[Concatenate[_Self, _Args], _Self]:
    @decorator
    def _generative(
        fn: Callable[Concatenate[_Self, _Args], None],
        self: _Self,
        *args: _Args.args,
        **kw: _Args.kwargs,
    ) -> _Self: ...

    decorated = _generative(fn)

    return decorated



# This sample tests a case where a method calls its own constructor
# with a specialized type that uses its own TypeVar and the expected
# type is a protocol.

from typing import Generic, TypeVar, Protocol

T = TypeVar("T")


class X(Protocol):
    def f(self) -> int: ...


class Y(Generic[T]):
    def f(self) -> T:
        raise NotImplementedError

    def g(self) -> X:
        # This should generate a type error.
        return Y[T]()



# This sample tests error detection of duplicate enum members and
# an attempt to subclass an enum.

from enum import Enum


class Color(Enum):
    red = "red"
    blue = "blue"
    yellow = "yellow"

    # This should generate an error because the enum member
    # already exists.
    blue = "blue"

    def __init__(self, value: str):
        if value == "blue":
            self.foo = False
        else:
            self.foo = True


class NonEnum: ...


# This should generate an error because enums cannot
# be subclassed.
class ExtraColor(NonEnum, Color):
    pass


# This should generate an error because reassignment of enum
# values is not allowed.
Color.red = "new"


class EnumWithoutValue(Enum):
    def do_something(self):
        pass

    @property
    def y(self) -> None:
        pass


class EnumWithValue(EnumWithoutValue):
    x = 0


# This should generate an error because enums with values
# cannot be subclassed.
class EnumSubclass(EnumWithValue):
    z: int



# This sample tests that type aliasing works.

from typing import Any, Literal

# Make sure it works with and without forward references.
TupleAlias = tuple["int", int]

v1: tuple[int, int]
v2: TupleAlias

v1 = (1, 2)
v2 = (1, 2)


AnyAlias = Any

v3: AnyAlias = 3


class A:
    Value1 = Literal[1]

    Value2 = 1


reveal_type(A.Value1, expected_text="type[Literal[1]]")
reveal_type(A.Value2, expected_text="int")


def func1(x: A.Value1):
    reveal_type(x, expected_text="Literal[1]")


Alias1 = Literal[0, 1]

v4: dict[Alias1, Any] = {}

if v4:
    pass

v5: list[Alias1] = []


Alias2 = int | str
Alias3 = int
Alias4 = type[int]


def func2(x: Alias2):
    reveal_type(type(x), expected_text="type[int] | type[str]")


def func3(v2: type[Alias2], v3: type[Alias3], v4: type[Alias4]):
    reveal_type(v2, expected_text="type[int] | type[str]")
    reveal_type(v3, expected_text="type[int]")
    reveal_type(v4, expected_text="type[type[int]]")


class B:
    TA1 = list

    def __init__(self) -> None:
        self.val = self.TA1


b = B()
reveal_type(b.val, expected_text="type[list[Unknown]]")



# This sample tests the handling of generic callbacks passed to a higher-order
# function that is also generic.

from typing import Callable, ParamSpec, Protocol, TypeVar

_T = TypeVar("_T")
_T1 = TypeVar("_T1")
_T_co = TypeVar("_T_co", covariant=True)
_U = TypeVar("_U")


class MyIterable(Protocol[_T_co]): ...


class MySupportsAbs(Protocol[_T_co]): ...


def my_abs(x: MySupportsAbs[_T], /) -> _T: ...


def my_map(a: Callable[[_T], _U], b: MyIterable[_T]) -> MyIterable[_U]: ...


def func1(xs: MyIterable[MySupportsAbs[int]]):
    ys0 = my_map(a=my_abs, b=xs)
    reveal_type(ys0, expected_text="MyIterable[int]")

    ys1 = my_map(b=xs, a=my_abs)
    reveal_type(ys1, expected_text="MyIterable[int]")


def ident(x: _U) -> _U:
    return x


def func2(__cb: Callable[[_T1], _T], __arg0: _T1) -> _T: ...


x1_0 = func2(ident, "hi")
reveal_type(x1_0, expected_text="str")

x1_1 = func2(ident, 1)
reveal_type(x1_1, expected_text="int")


_P = ParamSpec("_P")
_R = TypeVar("_R")


def func3(__obj: Callable[_P, _R], *args: _P.args, **kwargs: _P.kwargs) -> _R: ...


x2_0 = func3(ident, "hi")
reveal_type(x2_0, expected_text="str")

x2_1 = func3(ident, 1)
reveal_type(x2_1, expected_text="int")



# This sample tests basic usage of "closed" TypedDict classes as
# introduced in PEP 728.

from typing import TypedDict, Unpack


class Movie1(TypedDict, extra_items=int):
    name: str


def func1(movie: Movie1) -> None:
    del movie["year"]

    # This should generate an error.
    del movie["name"]


class Movie2(TypedDict, extra_items=int):
    name: str


def func2(**kwargs: Unpack[Movie2]) -> None: ...


func2(name="")

func2(name="", foo=1)

# This should generate an error.
func2(name=1)

# This should generate an error.
func2(name="", foo="")


m1 = Movie1(name="ET", year=1984)

# This should generate an error.
m2 = Movie1(name="ET", year="1984")



# This sample tests the case where a recursive type alias
# indirectly refers to itself through either a parameter
# annotation or return type annotation.

_str = str

def str(val: float) -> _str: ...

_int = int

def int(val: _int) -> None: ...



# This sample tests a piece of code that involves lots
# of cyclical dependencies for type resolution.


n: str | None = None
while True:
    if n is None:
        n = ""
    else:
        n = n + ""



from abc import ABC, abstractmethod, ABCMeta
from typing import Protocol, override

class A(ABC): 
    @abstractmethod
    def asdf(self) -> int: ...

class B(A): # error
    pass

class C(A, metaclass=ABCMeta): 
    pass

class D(A, ABC):
    pass

class E(A):
    @override
    def asdf(self) -> int:
        ...

class F(Protocol):
    @abstractmethod
    def asdf(self) -> int: ...

class G(F): # error
    pass

class H(F, Protocol):
    pass


# This sample tests the access of a ClassVar that uses Self in its
# declaration.

# It's not clear whether this should be permitted. Arguably, it's not
# type safe, but mypy admits it. This should be clarified in the typing
# spec.

from typing import ClassVar, Self


class Parent:
    x: ClassVar[dict[str, Self]] = {}

    @classmethod
    def __init_subclass__(cls):
        cls.x = {}



# This sample tests the handling of constrained type variables
# that include unions.

from typing import TypeVar

T1 = TypeVar("T1", int, str)
T2 = TypeVar("T2", int, str, int | str)
T3 = TypeVar("T3", int, str, int | str | list[int])


def func1(x: T1) -> T1:
    return x


def func2(x: T2) -> T2:
    return x


def func3(x: T3) -> T3:
    return x


def func4(y: int | str):
    # This should generate an error because T1 doesn't
    # include a union constraint.
    func1(y)

    func2(y)

    func3(y)



# This sample tests basic type narrowing behavior for
# the isinstance call.

from types import NoneType
from typing import Any, TypedDict


def func1(x: list[str] | int):
    if isinstance(x, list):
        reveal_type(x, expected_text="list[str]")
    else:
        reveal_type(x, expected_text="int")


def func2(x: Any):
    if isinstance(x, list):
        reveal_type(x, expected_text="list[Unknown]")
    else:
        reveal_type(x, expected_text="Any")


def func3(x):
    if isinstance(x, list):
        reveal_type(x, expected_text="list[Unknown]")
    else:
        reveal_type(x, expected_text="Unknown")


class SomeTypedDict(TypedDict):
    name: str


def func4(x: int | SomeTypedDict):
    if isinstance(x, dict):
        reveal_type(x, expected_text="SomeTypedDict")
    else:
        reveal_type(x, expected_text="int")


def func5(x: int | str | complex):
    if isinstance(x, (int, str)):
        reveal_type(x, expected_text="int | str")
    else:
        reveal_type(x, expected_text="complex | float")


def func6(x: type[int] | type[str] | type[complex]):
    if issubclass(x, (int, str)):
        reveal_type(x, expected_text="type[int] | type[str]")
    else:
        reveal_type(x, expected_text="type[complex] | type[float]")


def func7(x: int | SomeTypedDict | None):
    if isinstance(x, (dict, type(None))):
        reveal_type(x, expected_text="SomeTypedDict | None")
    else:
        reveal_type(x, expected_text="int")


def func8(x: type[int] | type[SomeTypedDict] | type[None]):
    if issubclass(x, (dict, type(None))):
        reveal_type(x, expected_text="type[SomeTypedDict] | type[None]")
    else:
        reveal_type(x, expected_text="type[int]")


def func9(x: int | None):
    if isinstance(x, NoneType):
        reveal_type(x, expected_text="None")
    else:
        reveal_type(x, expected_text="int")


def func10(x: type[int] | type[None]):
    if issubclass(x, NoneType):
        reveal_type(x, expected_text="type[None]")
    else:
        reveal_type(x, expected_text="type[int]")



# This sample tests the case where a recursive type alias previous
# led to infinite recursion.

from typing import Iterable, TypeVar, Union

T = TypeVar("T")
Tree = list[Union["Tree[T]", T]]


def _flatten(tree: Union[Tree[T], T]) -> Iterable[T]:
    if not isinstance(tree, list):
        yield tree
        return
    for v in tree:
        yield from _flatten(v)


def flatten(tree: Tree[T]) -> Iterable[T]:
    return _flatten(tree)


flatten([1, [2, 3]])



# This sample tests the case where a TypeVarTuple is used in a
# nested callable type.

from typing import Callable, TypeVarTuple

Ts = TypeVarTuple("Ts")


def func1(g: Callable[[Callable[[*Ts], None]], None]) -> tuple[*Ts]: ...


def func2(cb: Callable[[bytes, int], None]) -> None: ...


reveal_type(func1(func2), expected_text="tuple[bytes, int]")



# This sample tests the case where bidirectional type inference is required
# for the extra_items in a closed TypedDict.

from typing_extensions import TypedDict  # pyright: ignore[reportMissingModuleSource]


class Typed(TypedDict, extra_items=str | int):
    type: str


class Named(TypedDict, extra_items="str | int | Typed | Named"):
    name: str


td2_1: Named = {
    "name": "Fred",
    "birth": {
        "type": "date",
        "year": 2000,
        "month": 12,
        "day": 31,
    },
}

td2_2: Named = {
    "name": "Fred",
    "extra": {
        "name": "test",
        "value": "",
    },
}

td2_3: Named = {
    "name": "Fred",
}

td2_4: Named = {
    "name": "Fred",
    "test1": 1,
    "test2": {"name": "Barb", "value": {"type": "date", "day": 31}},
}



# This sample tests proper type narrowing within a double loop.

# pyright: strict

from typing import Callable


def func(call: Callable[[], None] | None):
    while True:
        while True:
            if call is None or call():
                break



from abc import ABC, abstractmethod
from typing import Protocol


class A:
    @abstractmethod
    def foo(self): ... # error


class B(A):
    @abstractmethod
    def asdf(self): ... # error


class C(ABC):
    @abstractmethod
    def foo(self): ...


class D(ABC):
    @abstractmethod
    def foo(self): ...


class E(Protocol):
    @abstractmethod
    def foo(self): ... # error (Protocols are a completely different concept to abstract classes, despite what the implementation of them will have you believe)


# This sample tests that generic type aliases are properly flagged as
# partially-unknown types if their type arguments are omitted.

# pyright: reportUnknownParameterType=true, reportMissingTypeArgument=false

from typing import TypeVar

T = TypeVar("T")
TA1 = list[T]


# This should generate an error because Foo is missing a type argument,
# so the type of `f` is partially unknown.
def func1(f: TA1) -> None:
    pass


TA2 = TA1


# This should generate an error because Bar doesn't specialize
# Foo appropriately.
def func2(f: TA2) -> None:
    pass


K = TypeVar("K")
V = TypeVar("V")

TA3 = dict[K, V]


# This should generate two errors because Baz is only partially specialized.
def func3(f: TA3[int]) -> None:
    pass



# This sample tests that enum values are treated as constant even if
# they are not named as such.

from enum import Enum


class EnumA(Enum):
    bad = 0
    good = 1


class EnumB:
    def __init__(self):
        self.status = EnumA.bad
        self.foo = 1


myobj = EnumB()

reveal_type(myobj.status, expected_text="EnumA")

myobj.status = EnumA.good
reveal_type(myobj.status, expected_text="Literal[EnumA.good]")



# This sample tests various function type checking
# behavior including arg/param matching.

from typing import Callable

# ------------------------------------------------------
# Test function type matching


class FooBase:
    pass


class Foo(FooBase):
    pass


class Bar(Foo):
    pass


def needs_function1(callback: Callable[[Foo], Foo]):
    pass


def callback1():
    pass


def callback2(a: Foo) -> Foo:
    return Foo()


def callback3(a: Foo) -> str:
    return "1"


def callback4(a: Foo, b: Foo) -> Foo:
    return Foo()


def callback5(a: Foo, b: int = 3) -> Foo:
    return Foo()


def callback6(*a) -> Foo:
    return Foo()


def callback7(a: str) -> Foo:
    return Foo()


def callback8(a: Bar) -> Foo:
    return Foo()


def callback9(a: FooBase) -> Foo:
    return Foo()


# This should generate an error because callback1
# takes no parameters.
needs_function1(callback1)

needs_function1(callback2)

# This should generate an error because the return
# type of callback3 doesn't match.
needs_function1(callback3)

# This should generate an error because callback4
# takes too many parameters.
needs_function1(callback4)

needs_function1(callback5)
needs_function1(callback6)

# This should fail because the parameter is the
# wrong type.
needs_function1(callback7)

# This should fail because the parameter is the
# wrong type.
needs_function1(callback8)

needs_function1(callback9)


import typing

# This should generate an error because modules are not callable.
typing()



# This sample tests that various types can be assigned to Type[Any].

from typing import Any, Type, TypeVar


class ClassA: ...


T = TypeVar("T")


def func1(x: Type[Any], y: Type[T]) -> T:
    v1: Type[Any] = x
    v2: Type[Any] = ClassA
    v3: Type[Any] = y

    return y()



# This sample tests the case where a specialized generic class that uses
# a ParamSpec and a callback protocol is assigned to a Callable that
# uses a ParamSpec.

from typing import Callable, Generic, TypeVar
from typing_extensions import (  # pyright: ignore[reportMissingModuleSource]
    Concatenate,
    ParamSpec,
)

P = ParamSpec("P")
R = TypeVar("R")


class MyPartial(Generic[P, R]):
    def __init__(self, first: int, func: Callable[Concatenate[int, P], R]) -> None:
        self.first = first
        self.func = func

    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> R: ...


class MyPartialCreator(Generic[P, R]):
    def __init__(self, func: Callable[Concatenate[int, P], R]):
        self.func = func

    def create_partial(self, first: int) -> Callable[P, R]:
        return MyPartial[P, R](first=first, func=self.func)



# This sample tests the case where a function parameterized with a
# ParamSpec P is called with *args: P.args and **kwargs: P.kwargs.

from typing import Any, Generic, ParamSpec

P = ParamSpec("P")


class TaskDeclaration(Generic[P]):
    pass


class Dispatcher:
    def dispatch(
        self,
        task_declaration: TaskDeclaration[P],
        count: int,
        /,
        *args: P.args,
        **kwargs: P.kwargs,
    ) -> None:
        pass


class Queue:
    dispatcher: Dispatcher

    def method1(self, stub: TaskDeclaration[P]) -> Any:
        def inner0(*args: P.args, **kwargs: P.kwargs) -> None:
            self.dispatcher.dispatch(stub, 1, *args, **kwargs)

        def inner1(*args: P.args, **kwargs: P.kwargs) -> None:
            # This should generate an error because a positional argument
            # cannot appear after an unpacked keyword argument.
            self.dispatcher.dispatch(stub, 1, **kwargs, *args)

        def inner2(*args: P.args, **kwargs: P.kwargs) -> None:
            # This should generate an error because it's missing
            # a positional argument for 'count'.
            self.dispatcher.dispatch(stub, *args, **kwargs)

        def inner3(*args: P.args, **kwargs: P.kwargs) -> None:
            # This should generate an error because it has an
            # additional positional argument.
            self.dispatcher.dispatch(stub, 1, 1, *args, **kwargs)

        def inner4(*args: P.args, **kwargs: P.kwargs) -> None:
            # This should generate an error because it is missing
            # the *args argument.
            self.dispatcher.dispatch(stub, 1, **kwargs)

        def inner5(*args: P.args, **kwargs: P.kwargs) -> None:
            # This should generate an error because it is missing
            # the *kwargs argument.
            self.dispatcher.dispatch(stub, 1, *args)

        def inner6(*args: P.args, **kwargs: P.kwargs) -> None:
            # This should generate an error because it has an
            # extra *args argument.
            self.dispatcher.dispatch(stub, 1, *args, *args, **kwargs)

            # This should generate an error because it has an
            # extra **kwargs argument.
            self.dispatcher.dispatch(stub, 1, *args, **kwargs, **kwargs)



# This sample tests the interaction between a traditional type alias
# and a PEP 695 type alias.

from typing import Annotated, TypeVar


T = TypeVar("T")

TA1 = Annotated[T, "metadata"]
TA2 = list[T]

type TA1_1 = TA1[int]
type TA1_2[T] = TA1[T]

type TA2_1 = TA2[int]
type TA2_2[T] = TA2[T]



# This sample tests an interaction between a lambda, a function
# that is declared later in the source, and a function return type
# that uses code flow analysis for evaluation.

from typing import Any, Callable, overload


class C: ...


@overload
def func1(v: Callable[[], int]) -> int: ...


@overload
def func1(v: Callable[[], list[C]]) -> list[C]: ...


def func1(v: Any) -> Any: ...


def func2(v: list[C]): ...


t = func1(lambda: func3())
t.append(C())


def func3() -> list[C]: ...



# This sample tests the incorrect usage of Union types.

from typing import Union

x = Union[int, str]

y = Union[int]

z = Union

# This should generate an error.
v1: Union[int]


# This should generate an error.
def func1() -> Union: ...


# This should generate an error.
var1: Union


# This should generate two errors.
def func2(x: (list | set)[int]):
    reveal_type(x)


# This should generate two errors.
def func3(x: Union[list, set][int]):
    reveal_type(x)



from collections.abc import Callable
from typing import Protocol, assert_type, dataclass_transform
from dataclasses import dataclass


@dataclass_transform(kw_only_default=True)
def rescoped[T: type](*, frozen: bool = False) -> Callable[[T], T]:
    return dataclass(frozen=frozen)  # pyright: ignore[reportUnknownVariableType]

@rescoped()
class Foo:
    x: str
    y: int

FOO = Foo(x="a", y=42)  # works


class Decorator(Protocol):
    def __call__[T: type](self, t: T, /) -> T:
        ...

@dataclass_transform(kw_only_default=True)
def spec(*, frozen: bool = False) -> Decorator:
    return dataclass(frozen=frozen)  # pyright: ignore[reportUnknownVariableType]

@spec()
class Bar:
    x: str
    y: int

BAR = Bar(x="a", y=42)

_ = assert_type(BAR, Bar)


# This sample tests the case where a type alias is used in a class
# declaration. We want to ensure that the variance of type variables
# is compatible with the usage within the type alias.

from typing import Generic, TypeVar, TypeAlias

T1 = TypeVar("T1")
T2 = TypeVar("T2", covariant=True)
T3 = TypeVar("T3", contravariant=True)


class A(Generic[T1]):
    pass


A_Alias_1: TypeAlias = A[T2]

A_Alias_2: TypeAlias = A_Alias_1[T2 | int]


# This should generate an error because the variance is incompatible.
class A_1(A_Alias_1[T2]): ...


# This should generate an error because the variance is incompatible.
class A_2(A_Alias_2[T2]): ...


# This should generate an error because the variance is incompatible.
class A_3(A[T2]): ...


class B(Generic[T1, T2]):
    pass


B_Alias_1 = B[T2, T3]


# This should generate an error because the variance is incompatible.
class C(B_Alias_1[T3, T2]): ...



# This sample tests the handling of a class with a custom
# __class_getitem__ class method.


from typing import Generic, TypeVar


class ClassA:
    # This should generate a warning because __class_getitem__
    # is implicitly a classmethod and should use cls rather than
    # self.
    def __class_getitem__(self, args: tuple[int, ...]) -> None: ...


reveal_type(ClassA[10, 63], expected_text="type[ClassA]")


_T = TypeVar("_T")
_S = TypeVar("_S")


class ClassB(Generic[_T, _S]):
    # Even though this class has a __class_getitem__ method,
    # it will be assumed to follow normal generic class semantics.
    def __class_getitem__(cls, args: tuple[int, ...]) -> None: ...


reveal_type(ClassB[int, str], expected_text="type[ClassB[int, str]]")



# This sample tests the reportFunctionMemberAccess diagnostic rule.


def func1():
    pass


a = func1.__annotations__
b = func1.__class__

# This should generate an error
c = func1.bar

# This should generate an error
func1.baz = 3

# This should generate an error
del func1.baz



# This sample tests the # pyright: ignore comment.

import sys


def func1(self, x: int | None) -> str:
    # This should suppress the error
    x + "hi"  # pyright: ignore - test

    # This should not suppress the error because the rule doesn't match.
    return 3  # pyright: ignore [foo]


if sys.version_info < (3, 8):
    x = 3  # pyright: ignore



# This sample tests the handling of instance variables within
# an enum class that are initialized by a custom initializer.
# They should not be treated as enum objects.

from enum import Enum


class Descriptor:
    def __get__(self, instance, owner=None) -> complex:
        return 3j


class EnumA(Enum):
    ENTRY = (123, "abc")

    desc = Descriptor()

    _exempt_ = 12

    foo: int
    bar: str

    def __init__(self, foo: int, bar: str) -> None:
        self.foo = foo
        self.bar = bar


baz = 123 + EnumA.ENTRY.foo
reveal_type(baz, expected_text="int")

reveal_type(EnumA._exempt_, expected_text="int")

reveal_type(EnumA.desc, expected_text="complex")



# This sample tests bidirectional type inference when the RHS
# is a call to a constructor.

from typing import (
    Any,
    Generic,
    Iterable,
    Literal,
    Mapping,
    Protocol,
    Self,
    TypeVar,
)


_T1 = TypeVar("_T1")
_T1_contra = TypeVar("_T1_contra", contravariant=True)
_T2 = TypeVar("_T2")
_T3 = TypeVar("_T3")


class Animal(Generic[_T1, _T2]):
    pass


class Bear(Animal[_T3, int]):
    def __init__(self, p1: _T3 | None = None):
        pass


class Donkey(Animal[int, int], Generic[_T3]):
    pass


class Flyer(Protocol[_T1_contra]):
    def get_wingspan(self, p1: _T1_contra) -> float:
        raise NotImplemented


class CaveDweller(Generic[_T1]):
    pass


class Bat(Animal[int, int], CaveDweller[int]):
    def get_wingspan(self, p1: int) -> float:
        raise NotImplemented


def s1():
    b: Bear[str] = Bear()
    a: Animal[str, int] = b
    reveal_type(a, expected_text="Bear[str]")


def s2():
    a: Animal[str, int] = Bear()
    reveal_type(a, expected_text="Bear[str]")


def s3():
    a: Animal[str, int] = Bear()
    reveal_type(a, expected_text="Bear[str]")


def s4():
    a: Bear[Any] = Bear[int]()
    reveal_type(a, expected_text="Bear[Any]")


def s5():
    a: Animal[Any, Any] = Bear[int]()
    reveal_type(a, expected_text="Bear[int]")


def s6():
    a: Bat | Bear[str] = Bear()
    reveal_type(a, expected_text="Bear[str]")


def s7(p: Bat | Bear[int]):
    a: Animal[int, int] = p
    reveal_type(a, expected_text="Bat | Bear[int]")


def s8():
    a: Animal[int, int] = Bear[int]()
    reveal_type(a, expected_text="Bear[int]")


def s9(p: dict[str, str]):
    a: dict[str, Any] = p
    reveal_type(a, expected_text="dict[str, Any]")


def s10(p: list[str]):
    a: Iterable[Any] = p
    reveal_type(a, expected_text="list[str]")
    b: Iterable[str] = []
    reveal_type(b, expected_text="list[str]")
    c: Iterable[str] = list()
    reveal_type(c, expected_text="list[str]")


def s11():
    a: Animal[Any, Any] = Donkey[int]()
    reveal_type(a, expected_text="Donkey[int]")


def s12(p: Bear[_T1], b: _T1):
    a: Animal[Any, int] = p
    reveal_type(a, expected_text="Bear[_T1@s12]")


def s13(p: Bat):
    a: Flyer[int] = p
    reveal_type(a, expected_text="Bat")


def s14(p: Bat):
    a: CaveDweller[int] = p
    reveal_type(a, expected_text="Bat")


def s15():
    a = Bear(1)
    reveal_type(a, expected_text="Bear[int]")
    b = Bear[int](1)
    reveal_type(b, expected_text="Bear[int]")
    c = Bear[float](1)
    reveal_type(c, expected_text="Bear[float]")
    d = Bear[str | int](1)
    reveal_type(d, expected_text="Bear[str | int]")


def s16():
    a: Any = Bear(1)
    reveal_type(a, expected_text="Any")


def s17():
    a1: Iterable[object] = [2, 3, 4]
    reveal_type(a1, expected_text="list[int]")

    a2: list[object] = [2, 3, 4]
    reveal_type(a2, expected_text="list[object]")

    b1: Iterable[float] = [2, 3, 4]
    reveal_type(b1, expected_text="list[int]")

    b2: list[float] = [2, 3, 4]
    reveal_type(b2, expected_text="list[float]")

    c1: Iterable[Literal["A", "B", "C"]] = ["A", "B"]
    reveal_type(c1, expected_text="list[Literal['A', 'B']]")

    c2: list[Literal["A", "B", "C"]] = ["A", "B"]
    reveal_type(c2, expected_text="list[Literal['A', 'B', 'C']]")


def s18():
    a1: Mapping[object, object] = {"a": 3, "b": 5.6}
    reveal_type(a1, expected_text="dict[object, int | float]")

    a2: dict[object, object] = {"a": 3, "b": 5.6}
    reveal_type(a2, expected_text="dict[object, object]")

    b1: Mapping[str, float] = {"a": 3, "b": 5}
    reveal_type(b1, expected_text="dict[str, int]")

    b2: dict[str, float] = {"a": 3, "b": 5}
    reveal_type(b2, expected_text="dict[str, float]")

    c1: Mapping[Literal["A", "B"], Literal[3, 4]] = {"A": 3}
    reveal_type(c1, expected_text="dict[Literal['A', 'B'], Literal[3]]")

    c2: dict[Literal["A", "B"], Literal[3, 4]] = {"A": 3}
    reveal_type(c2, expected_text="dict[Literal['A', 'B'], Literal[3, 4]]")


class Plant(Generic[_T1]):
    def __new__(cls, o: _T1) -> Self: ...


plant: Plant[float] = Plant(0)



# This sample tests a protocol that uses generics in the "self"
# parameter.

from typing import Protocol, Self, TypeVar

T = TypeVar("T")


class HasParent(Protocol):
    def get_parent(self: T) -> T: ...


GenericNode = TypeVar("GenericNode", bound=HasParent)


def generic_get_parent(n: GenericNode) -> GenericNode:
    return n.get_parent()


class ConcreteNode:
    def get_parent(self) -> Self:
        return self


node = ConcreteNode()
parent = generic_get_parent(node)



# This sample tests the handling of the "super" call when
# used with a two-argument form that specifies the "bind to" type.

from typing import TypeVar

T = TypeVar("T", bound="A")


class A:
    def __init__(self, **kw: object) -> None:
        pass

    @classmethod
    def factoryA(cls: type[T]) -> T:
        return cls()

    @classmethod
    def get(cls: type[T], key: str) -> T:
        return cls()


class B(A):
    @classmethod
    def factoryB(cls):
        return super(B, cls).factoryA()

    @classmethod
    def get(cls, key: str = ""):
        return super(B, cls).get(key)


class BChild(B):
    pass


a1 = A.factoryA()
reveal_type(a1, expected_text="A")

b1 = B.factoryA()
reveal_type(b1, expected_text="B")

b2 = B.factoryB()
reveal_type(b2, expected_text="B")

g1 = B.get()
reveal_type(g1, expected_text="B")

g2 = BChild.get()
reveal_type(g2, expected_text="BChild")


def test_a(cls: type[T]) -> T:
    return super(A, cls).__new__(cls)


class C:
    def __init__(self) -> None: ...


class CChild(C):
    def __init__(self, name: str) -> None: ...


class D:
    def __init__(self, name: str, num: int): ...


class DChild1(CChild, D):
    def __init__(self, name: str, num: int) -> None:
        super(C, self).__init__(name, num)


class DChild2(CChild, D):
    def __init__(self, name: str) -> None:
        super(DChild2, self).__init__(name)


class DChild3(CChild, D):
    def __init__(self) -> None:
        super(CChild, self).__init__()


d1 = DChild1("", 1)
d2 = DChild2("")
d3 = DChild3()


class E:
    def __new__(cls) -> "E":
        return super(type, cls).__new__(cls)


class F: ...


class FChild1(F): ...


def func1(cls: type[F | FChild1]):
    super(F, cls)



# This sample tests tuple parameter matching for
# cases where an unpack operator is used in the target.

from typing import Iterable

a: int
b: int
c: str
d: str
e: Iterable[int]
f: Iterable[str | int]

# This should generate an error because an unpack
# operator must be within a tuple.
*e = 3, 4, 5, 6

(*e,) = 3, 4, 5, 6

a, b, *e, c, d = 3, 4, "a", "b"

a, b, *f, c, d = 3, 4, 5, "a", "b", "c"

*f, a, b, c, d = 3, 4, "a", "b"
a, *f, b, c, d = 3, 4, "a", "b"
a, b, *f, c, d = 3, 4, "a", "b"
a, b, c, *f, d = 3, 4, "a", "b"
a, b, c, d, *f = 3, 4, "a", "b"

a, b, c, *f = 3, 2, ""

# This should generate an error because there are
# not enough source values.
*f, a, b, c = 3, 2
a, *f, b, c = 3, 2
a, b, *f, c = 3, 2
a, b, c, *f = 3, 2

# This should generate an error because there are
# too many source values.
a, b = 3, 2, 3

# This should generate an error because e can't
# accommodate both int and str types.
a, b, *e, c, d = 3, 4, 5, "a", "b", "c"


def func1(p1: tuple[str, ...]):
    global a, b, c, d

    c, d = p1

    # This should generate an error because
    # p1 is an incompatible type.
    a, b = p1

    c, d, *f = p1


def func2(p1: tuple[str, ...], p2: tuple[str, *tuple[str, ...]]):
    () = p1
    (_,) = p1
    (_, _) = p1

    # This should generate an error.
    () = p2
    (_,) = p2



# This sample tests the case where a generic alias refers to a Callable
# type and the alias is used without type arguments.

from typing import Callable, ParamSpec, TypeVar

T = TypeVar("T")
P = ParamSpec("P")

TA1 = Callable[[T], T]
TA2 = Callable[[T], T] | Callable[P, T]


def f1() -> TA1: ...


reveal_type(f1(), expected_text="(Unknown) -> Unknown")


def f2() -> TA2: ...


g2 = f2()
reveal_type(
    g2,
    expected_text="((Unknown) -> Unknown) | ((...) -> Unknown)",
)
reveal_type(g2(42), expected_text="Unknown")



# This sample tests the case where a NewType is used as a class pattern.

from typing import NewType


NT1 = NewType("NT1", int)


def accepts_widget_id(value: NT1 | int) -> None:
    match value:
        case NT1():
            # This should generate an error because NewType returns a function at runtime.
            pass


NT2 = NewType("NT2", str)


def accepts_union(value: NT1 | NT2) -> None:
    match value:
        case NT1():
            # This should generate an error because NewType returns a function at runtime.
            pass

        case NT2():
            # This should generate an error because NewType returns a function at runtime.
            pass



# This sample tests various conditions under which Unpack
# can and cannot be used.

# pyright: reportMissingModuleSource=false

from typing import Generic, TypeVar, Union
from typing_extensions import (  # pyright: ignore[reportMissingModuleSource]
    TypeVarTuple,
    Unpack,
)


_T = TypeVar("_T")
_Xs = TypeVarTuple("_Xs")


class ClassA(Generic[_T, Unpack[_Xs]]):
    def __init__(self, *shape: Unpack[_Xs]):
        self.x: tuple[Unpack[_Xs]] = shape

        # This should generate an error
        self.y: _Xs = shape

    # This should generate two errors
    def func1(self) -> Union[Unpack[_Xs]]: ...

    # This should generate an error
    def func2(self) -> tuple[Unpack[_T]]: ...

    # This should generate an error
    def func3(self) -> tuple[Unpack[int]]: ...

    # This should generate an error
    def func4(self) -> tuple[Unpack[_Xs, _Xs]]: ...

    # This should generate an error.
    a: list[Unpack[_Xs]] = []

    # This should generate an error.
    b: Unpack[_Xs] = ()


# This should generate an error.
x: list[Unpack[_Xs]] = []

# This should generate an error.
y: Unpack[_Xs] = ()

# This should generate an error.
z: Unpack = ()


class Array(Generic[Unpack[_Xs]]): ...


# This should generate two errors because _Xs must be unpacked.
def func0(value: Array[_Xs]) -> tuple[complex, _Xs, str]: ...



# This sample tests that type guards work correctly
# with the walrus operator.

import re


def foo(s: str) -> str:
    if m := re.fullmatch("(test).+", s):
        return m.group(1)
    return "oops"



# This sample tests the case where a generic type alias with a TypeVarTuple
# also contains other TypeVars, and it is specialized with an unpacked tuple.

from typing import TypeVar, TypeVarTuple

Ts = TypeVarTuple("Ts")
T1 = TypeVar("T1")
T2 = TypeVar("T2")

TA1 = tuple[T1, *Ts, T2]
TA1_Spec1 = TA1[*tuple[int, ...]]
TA1_Spec2 = TA1[float, *tuple[int, ...]]
TA1_Spec3 = TA1[*tuple[int, ...], str]
TA1_Spec4 = TA1[float, *tuple[int, ...], str]

TA2 = tuple[*Ts, T1, T2]
TA2_Spec1 = TA2[*tuple[int, ...]]

TA3 = tuple[T1, T2, *Ts]
TA3_Spec1 = TA3[*tuple[int, ...]]

TA4 = tuple[T1, T1, *Ts, T2, T2]
TA4_Spec1 = TA4[*tuple[int, ...]]


def func1(
    ta1_1: TA1_Spec1,
    ta1_2: TA1_Spec2,
    ta1_3: TA1_Spec3,
    ta1_4: TA1_Spec4,
    ta2: TA2_Spec1,
    ta3: TA3_Spec1,
    ta4: TA4_Spec1,
):
    reveal_type(ta1_1, expected_type="tuple[int, *tuple[int, ...], int]")
    reveal_type(ta1_2, expected_type="tuple[float, *tuple[int, ...], int]")
    reveal_type(ta1_3, expected_type="tuple[int, *tuple[int, ...], str]")
    reveal_type(ta1_4, expected_type="tuple[float, *tuple[int, ...], str]")
    reveal_type(ta2, expected_type="tuple[*tuple[int, ...], int, int]")
    reveal_type(ta3, expected_type="tuple[int, int, *tuple[int, ...]]")
    reveal_type(ta4, expected_type="tuple[int, int, *tuple[int, ...], int, int]")



from _typeshed import Incomplete
from typing import TypeVar

from pygments.formatter import Formatter

_T = TypeVar("_T", str, bytes)

class RtfFormatter(Formatter[_T]):
    name: str
    aliases: Incomplete
    filenames: Incomplete
    fontface: Incomplete
    fontsize: Incomplete
    def format_unencoded(self, tokensource, outfile) -> None: ...



# This sample tests the case where a class defines a __new__ method
# that returns a class other than the owning class.


class A:
    def __init__(self) -> None:
        pass


class B(A):
    def __new__(cls) -> A:
        return A()

    def __init__(self, a: int) -> None:
        pass


class C(B):
    def __init__(self, a: int) -> None:
        pass


B()

# This should generate an error because B.__init__ is never called.
B(1)

C()

# This should generate an error because C.__init__ is never called.
C(1)



# This sample tests the handling of protocol classes that define properties
# to indicate a read-only attribute. It also tests that a member access through
# a protocol class (not an instance) is flagged as an error.

from typing import Protocol


class A(Protocol):
    @property
    def name(self) -> str: ...


class B:
    name: str


def do_something(a: A, class_a: type[A]) -> None:
    val1 = a.name
    reveal_type(val1, expected_text="str")

    # This should generate an error because accesses to
    # properties from a protocol class are not allowed.
    val2 = class_a.name

    val3: A = B()



# This sample tests the type checker's ability to handle
# class variables that redefine a symbol in an outer
# scope but are not defined with an explicit class
# variable statement.


class ClassA:
    bar: str = "hi"

    def __init__(self, val: str) -> None:
        self.str = val

    @classmethod
    def method1(cls, val: str) -> None:
        cls.str = val



# This sample tests the case where a dataclass converter is used with
# a generic type.

from typing import Any, Callable, dataclass_transform


def model_field(*, converter: Callable[..., Any]) -> Any: ...


@dataclass_transform(field_specifiers=(model_field,))
class ModelBase: ...


class DC1[T](ModelBase):
    data: set[T] = model_field(converter=set)


x = DC1([1, 2])
reveal_type(x, expected_text="DC1[int]")
reveal_type(x.data, expected_text="set[int]")



# This sample tests dictionary inference logic.

from typing import Mapping, TypeAlias, TypeVar

T = TypeVar("T")


def func1(mapping: Mapping[str | bytes, int]):
    return mapping


func1({"x": 1})
func1({b"x": 1})

# This should generate an error.
func1({3: 1})


RecursiveMapping: TypeAlias = (
    int | Mapping[int, "RecursiveMapping"] | Mapping[str, "RecursiveMapping"]
)


class HasName:
    name: str | None


def func2(x: T | None) -> T:
    assert x is not None
    return x


def func3(v: list[HasName]) -> RecursiveMapping:
    return {func2(x.name): 1 for x in v}



# This sample tests the specialization of a property
# provided by a generic subclass.

from typing import TypeVar, Generic

T = TypeVar("T", bound=int)


class Foo(Generic[T]):
    def __init__(self, bar: T):
        self._bar = bar

    @property
    def bar(self) -> T:
        return self._bar

    def bar_method(self) -> T:
        return self._bar


class NewInt(int):
    def new_thing(self):
        pass


class FooNewInt(Foo[NewInt]):
    def fizz(self) -> None:
        self.bar.new_thing()
        self.bar_method().new_thing()



# This sample tests that a generic type alias retains a literal type argument
# when it is specialized.

from typing import Literal, TypeAlias, TypeVar

Mode = Literal["read", "write"]
T = TypeVar("T")
Entry: "TypeAlias" = dict[T, int]
Entry2: TypeAlias = dict[Mode, int]


def f() -> Entry[Mode]:
    return {"read": 0}


def g() -> Entry2:
    return {"read": 0}


def main() -> None:
    d1 = f()
    reveal_type(d1, expected_text="dict[Literal['read', 'write'], int]")
    d2 = g()
    reveal_type(d2, expected_text="dict[Literal['read', 'write'], int]")



# This sample tests the case where a property is defined on a metaclass.


class MyMeta(type):
    @property
    def something(cls) -> "Base":
        return Base(1234)


class Base(metaclass=MyMeta):
    def __new__(cls, arg) -> "Base": ...


reveal_type(Base.something, expected_text="Base")



# This sample tests that a Generic base class overrides the type parameter
# ordering of other type parameters.

# pyright: reportIncompatibleMethodOverride=false

from typing import Generic, Iterable, Iterator, Mapping, TypeVar

_T1 = TypeVar("_T1")
_T2 = TypeVar(
    "_T2", "str", "int"
)  # 'str' and 'int' should be treated as forward reference


class Foo(Iterable[_T2], Generic[_T1, _T2]):
    def __init__(self, a: _T1, b: _T2):
        pass

    def foo(self, a: _T1, b: _T2) -> _T2:
        return b

    def __iter__(self) -> Iterator[int]: ...


a: Foo[int, str] = Foo(2, "")
b: str = a.foo(4, "")


# This should generate an error because a class shouldn't
# derive from Generic more than once.
class Bar(Generic[_T1], Generic[_T2]):
    pass


K = TypeVar("K")
V = TypeVar("V")


# This should generate an error because V isn't included
# in the Generic type variable list.
class A(Mapping[K, V], Generic[K]): ...



# This sample tests the special-case handling for __slots__ and
# __class_getitem__ during protocol matching.


from typing import Any, Iterable, Protocol


class B: ...


class C:
    def __class_getitem__(cls, __item: Any) -> Any: ...


class SupportsClassGetItem(Protocol):
    __slots__: str | Iterable[str] = ()

    def __class_getitem__(cls, __item: Any) -> Any: ...


b1: SupportsClassGetItem = B()  # OK (missing __class_getitem__ is ignored)
c1: SupportsClassGetItem = C()  # OK


# This should generate an error because __class_getitem__ is not exempt
# when performing class object protocol matching.
b2: SupportsClassGetItem = B  # Error
c2: SupportsClassGetItem = C  # OK



# This sample tests that a class that derives from Any can be used
# to satisfy a TypeVar.

from typing import Any, TypeVar


T = TypeVar("T")


def foo(self, obj: T, foo: Any) -> T:
    # NotImplemented is an instance of a class that derives from Any.
    return NotImplemented



# This sample tests type inference for list comprehensions,
# including list target expressions in for statements.


def foo() -> list[str]:
    pairs = [s.split(":") if ":" in s else [s, "null"] for s in ["foo:bar", "baz"]]
    foo = [p[0] for p in pairs]
    if foo:
        return foo

    bar = [a for [a, b] in pairs]
    return bar



# This sample tests the case where a TypeVar is bound to a
# protocol class.

from typing import Protocol, TypeVar


class ClsProtocol(Protocol):
    def __init__(self): ...


T1 = TypeVar("T1", bound="ClsProtocol")


class Sample:
    @classmethod
    def test(cls: type[T1]) -> T1:
        return cls()


reveal_type(Sample.test(), expected_text="Sample")
reveal_type(Sample().test(), expected_text="Sample")



# This sample tests various type checking operations relating to
# async generator functions where the return type is inferred.

from typing import AsyncGenerator, Awaitable, Generator, Iterator


async def g1():
    yield 1
    yield 2


async def g2():
    async for v in g1():
        yield v


def g1_explicit1() -> Generator[int, None, None]:
    yield 1
    yield 2


async def g1_explicit2() -> AsyncGenerator[int, None]:
    yield 1
    yield 2


async def g2_explicit():
    for v in g1_explicit1():
        yield v

    async for v in g1_explicit2():
        yield v


async def g3(xs: Awaitable[list[int]]) -> list[int]:
    return [x for x in await xs]


async def g4(xs: list[Awaitable[int]]) -> list[int]:
    return [await x for x in xs]


class SomeIterable:
    def __init__(self):
        self.x = 1

    def __iter__(self) -> Iterator[int]:
        yield self.x


async def func1() -> SomeIterable:
    return SomeIterable()


def func2() -> Iterator[int]:
    yield 2


def g5() -> None:
    val = (y for y in func2())
    reveal_type(val, expected_text="Generator[int, None, None]")


async def g6() -> None:
    val = (x + y for y in func2() for x in await func1())
    reveal_type(val, expected_text="AsyncGenerator[int, None]")


async def g7() -> None:
    val = (x + y for y in await func1() for x in func2())
    reveal_type(val, expected_text="Generator[int, None, None]")



# This sample exercises the type analyzer's type narrowing
# logic for assert statements.

condition: bool = True


def func1(a: str | int) -> int:
    if condition:
        # This should generate an error because
        # a could be a str.
        return a

    assert isinstance(a, int)

    return a


def func2(a: str | int) -> int:
    # Test the form of "assert" that includes a message string.
    assert isinstance(a, int), "Message"
    return a



# This sample tests the type checker's validation of instance
# variables that are declared in the __slots__ attribute.


class NoSlots1:
    def __init__(self):
        self.x = 1


class NoSlots2:
    # Only lists and tuples of simple strings are supported, so this
    # will be treated as though there are no slots.
    __slots__ = {"aaa": 3}

    def __init__(self):
        self.x = 1


class NoSlots3:
    # Only lists and tuples of simple strings are supported, so this
    # will be treated as though there are no slots.
    __slots__ = ("aaa", f"test{3 + 4}")

    def __init__(self):
        self.x = 1


class Slots1(object):
    __slots__ = ("bbb", "ccc")

    def __init__(self):
        self.bbb = 1
        self.ccc = 1
        self.prop = 1

        # This should generate an error
        self.ddd = 1

    @property
    def prop(self):
        pass

    @prop.setter
    def prop(self, val: int):
        pass


class Slots1_1(Slots1):
    __slots__ = ["ddd", "eee"]

    def __init__(self):
        self.bbb = 1
        self.ccc = 1
        self.ddd = 1

        # This should generate an error
        self.fff = 1


class NoSlots1_1(Slots1, NoSlots2):
    def __init__(self):
        self.bbb = 1
        self.fff = 1



# This sample tests the type narrowing logic for
# conditional expression involving assignment expressions
# (walrus operator).

# pyright: strict


class C:
    def method1(self):
        pass


def good(b: C | None) -> None:
    a = b
    if a:
        a.method1()


def bad(b: C | None) -> None:
    if c := b:
        c.method1()
        b.method1()



import sys
from _typeshed import SupportsRichComparisonT
from asyncio.events import AbstractEventLoop
from types import GenericAlias
from typing import Any, Generic, TypeVar

if sys.version_info >= (3, 10):
    from .mixins import _LoopBoundMixin
else:
    _LoopBoundMixin = object

class QueueEmpty(Exception): ...
class QueueFull(Exception): ...

# Keep asyncio.__all__ updated with any changes to __all__ here
if sys.version_info >= (3, 13):
    __all__ = ("Queue", "PriorityQueue", "LifoQueue", "QueueFull", "QueueEmpty", "QueueShutDown")

else:
    __all__ = ("Queue", "PriorityQueue", "LifoQueue", "QueueFull", "QueueEmpty")

_T = TypeVar("_T")

if sys.version_info >= (3, 13):
    class QueueShutDown(Exception): ...

# If Generic[_T] is last and _LoopBoundMixin is object, pyright is unhappy.
# We can remove the noqa pragma when dropping 3.9 support.
class Queue(Generic[_T], _LoopBoundMixin):  # noqa: Y059
    if sys.version_info >= (3, 10):
        def __init__(self, maxsize: int = 0) -> None: ...
    else:
        def __init__(self, maxsize: int = 0, *, loop: AbstractEventLoop | None = None) -> None: ...

    def _init(self, maxsize: int) -> None: ...
    def _get(self) -> _T: ...
    def _put(self, item: _T) -> None: ...
    def _format(self) -> str: ...
    def qsize(self) -> int: ...
    @property
    def maxsize(self) -> int:
        """Number of items allowed in the queue."""
        ...
    def empty(self) -> bool: ...
    def full(self) -> bool: ...
    async def put(self, item: _T) -> None: ...
    def put_nowait(self, item: _T) -> None: ...
    async def get(self) -> _T: ...
    def get_nowait(self) -> _T: ...
    async def join(self) -> None: ...
    def task_done(self) -> None: ...
    def __class_getitem__(cls, type: Any, /) -> GenericAlias:
        """
        Represent a PEP 585 generic type

        E.g. for t = list[int], t.__origin__ is list and t.__args__ is (int,).
        """
        ...
    if sys.version_info >= (3, 13):
        def shutdown(self, immediate: bool = False) -> None: ...

class PriorityQueue(Queue[SupportsRichComparisonT]): ...
class LifoQueue(Queue[_T]): ...



# This sample tests the type handler's handling of the
# built-in NewType function.

from abc import ABC, abstractmethod
from typing import Any, NewType, TypeVar, TypedDict

MyString = NewType("MyString", "str")


def must_take_string(p1: str):
    pass


must_take_string(MyString("hello"))


def must_take_my_string(p1: MyString):
    pass


must_take_my_string(MyString("hello"))

# This should generate an error because 'hello'
# isn't a valid MyString.
must_take_my_string("hello")


_T = TypeVar("_T")


def func1(x: type[_T]) -> type[_T]:
    return x


MyString2 = NewType("MyString2", func1(str))

# This should generate an error because NewType requires two arguments.
NewTypeBad1 = NewType()

# This should generate an error because NewType requires two arguments.
NewTypeBad2 = NewType("NewTypeBad2")

# This should generate an error because NewType requires two arguments.
NewTypeBad3 = NewType("NewTypeBad3", int, int)

# This should generate an error because the first argument must be a string literal.
NewTypeBad4 = NewType(int, int)

args = ("Hi", int)
# This should generate an error because two positional args are needed.
NewTypeBad5 = NewType(*args)

# This should generate an error because type cannot be Any.
NewTypeBad6 = NewType("NewTypeBad6", Any)


class TD1(TypedDict):
    x: int


# This should generate an error because type cannot be a TypedDict.
NewTypeBad7 = NewType("NewTypeBad7", TD1)

NewTypeGood8 = NewType("NewTypeGood8", MyString)

# This should generate an error because the name doesn't match.
NewTypeBad9 = NewType("NewTypeBad9Not", int)


def func2(x: MyString):
    # This should generate two errors because isinstance can't be used
    # with a NewType and it violates the isinstance siganture.
    if isinstance(x, MyString):
        pass

    # This should generate two errors because issubclass can't be used
    # with a NewType and it violates the issubclass signature.
    if issubclass(type(x), (MyString, int)):
        pass


class AbstractBase(ABC):
    @abstractmethod
    def method1(self, /) -> int: ...


class DerivedBase(AbstractBase):
    def method1(self, /) -> int:
        return 0


NewDerived = NewType("NewDerived", AbstractBase)
new_derived = NewDerived(DerivedBase())



# This sample validates that parameter types specified for "self"
# and "cls" parameters are compatible with the containing class.

from typing import Iterator, TypeVar


class Parent:
    pass


_T = TypeVar("_T")
_TChild1 = TypeVar("_TChild1", bound="Child1")


class Child1:
    def m1(self: "Child1"): ...

    # This should generate an error.
    def m2(self: Parent): ...

    # This should generate an error.
    def m3(self: type["Child1"]): ...

    def m4(self: _TChild1) -> _TChild1: ...

    # This should generate an error.
    def m5(self: type[_TChild1]) -> _TChild1: ...

    def m6(self: _T) -> _T: ...

    # This should generate an error.
    def __new__(cls: "Child1"): ...

    @classmethod
    def cm1(cls: type["Child1"]): ...

    # This should generate an error.
    @classmethod
    # This should generate an error.
    def cm2(cls: "Child1"): ...

    @classmethod
    # This should generate an error.
    def cm3(cls: type[Parent]): ...

    @classmethod
    def cm4(cls: type[_TChild1]) -> _TChild1: ...

    # This should generate an error.
    @classmethod
    # This should generate an error.
    def cm5(cls: _TChild1) -> _TChild1: ...

    @classmethod
    def cm6(cls: type[_T]) -> _T: ...


class MyMeta(type):
    def m1(self: type[_T]) -> Iterator[_T]: ...



# This sample is similar to pseudoGeneric2.py in that it tests
# the case where the class's constructor is unannotated. This
# particular case was causing an internal crash.

import abc


class ClassB(metaclass=abc.ABCMeta):
    def __init__(self, value=None):
        self._cache = {"value": value}

    @property
    def cache(self):
        return self._cache

    @cache.deleter
    def cache(self):
        self._cache = {key: None for key in self._cache}

    def __getattr__(self, attr):
        cache = self.cache
        if attr in cache:
            return cache[attr]
        else:
            return self.__getattribute__(attr)


b1 = ClassB("test")
reveal_type(b1.value, expected_text="Unknown | Any | None")
del b1.cache



# This sample tests pyright's ability to use metaclasses.

from ctypes import Array, c_uint64
from typing import Any, TypeAlias, TypeVar

myArray1 = (c_uint64 * 5)()

myArray2: Array[c_uint64] = (c_uint64 * 5)()


T = TypeVar("T")


class CustomMeta(type):
    def __getitem__(self, key: Any) -> "type[int]": ...


class Custom(metaclass=CustomMeta): ...


class OtherMeta(type): ...


# This should generate an error because the class isn't
# Generic even though it supports a metaclass with a
# __getitem__.
y1: Custom[int]

# This should not generate an error because it is used
# as a runtime expression rather than a type annotation.
y2 = Custom[int]

# This should generate an error.
y3: TypeAlias = Custom[int]


def func1(m: CustomMeta):
    v1: type = m
    v2: type[Any] = m

    # This should generate an error.
    v3: type[Custom] = m


def func2(m: OtherMeta):
    # This should generate an error because OtherMeta
    # and the metaclass for Custom are not compatible.
    v3: type[Custom] = m



# This sample tests the handling of nested functions that involve ParamSpecs.

import functools
from typing import Callable, TypeVar
from typing_extensions import ParamSpec  # pyright: ignore[reportMissingModuleSource]

P = ParamSpec("P")
R = TypeVar("R")


def deprecated(
    instead: str | None = None,
) -> Callable[[Callable[P, R]], Callable[P, R]]:
    def actual_decorator(func: Callable[P, R]) -> Callable[P, R]:
        @functools.wraps(func)
        def decorated(*args: P.args, **kwargs: P.kwargs) -> R:
            return func(*args, **kwargs)

        return decorated

    return actual_decorator



# This sample tests synthesized get methods in TypedDict classes.

from typing import TypedDict

UserType1 = TypedDict("UserType1", {"name": str, "age": int}, total=False)
user1: UserType1 = {"name": "Bob", "age": 40}

name1: str = user1.get("name", "n/a")
age1: int = user1.get("age", 42)

UserType2 = TypedDict("UserType2", name=str, age=int)
user2: UserType2 = {"name": "Bob", "age": 40}

name2: str | None = user2.get("name")

name3: str = user2.get("name")

age2: int = user2.get("age", 42)

age3: int | str = user2.get("age", "42")

age4: int = user2.get("age", "42")



# This sample tests the use of a pyright ignore comment in conjunction
# with the reportUnnecessaryTypeIgnoreComment mechanism.


def func1(self, x: int | None) -> str:
    # This should suppress the error
    v1 = x + "hi"  # pyright: ignore - test

    # This is unnecessary
    v2 = x + x  # pyright: ignore

    # This will not suppress the error
    # These are both unnecessary
    v3 = x + x  # pyright: ignore [foo, bar]

    # This will not suppress the error
    # erm actually it does because it's now treated the same as an ignore comment with no codes soooo
    v4 = x + x  # pyright: ignore []

    # One of these is unnecessary
    v5 = x + "hi"  # test # pyright: ignore [reportOperatorIssue, foo]

    return 3  # pyright: ignore [reportReturnType]



# This sample tests the reportUnnecessaryComparison check when applied
# to match statements.

from typing import Literal, Mapping, Sequence

Letters = Literal["A", "B", "C"]


def func1(subj: Letters):
    match subj:
        # This should generate an error if reportUnnecessaryComparison is enabled.
        case "A" | "B" | "D":
            pass
        case str():
            pass
        # This should generate an error if reportUnnecessaryComparison is enabled.
        case "C":
            pass
        # This should generate an error if reportUnnecessaryComparison is enabled.
        case x:
            print(x)


def func2(subj: int | dict[str, str]):
    match subj:
        # This should generate an error if reportUnnecessaryComparison is enabled.
        case str() if subj > 4:
            pass
        case int() if subj > 4:
            pass
        case int():
            pass
        # This should generate an error if reportUnnecessaryComparison is enabled.
        case int():
            pass
        # This should generate an error if reportUnnecessaryComparison is enabled.
        case (a, b):
            print(a, b)
        case {"": d}:
            print(d)
        case dict():
            pass
        # This should generate an error if reportUnnecessaryComparison is enabled.
        case x:
            print(x)


JsonValue = (
    None | bool | int | float | str | Sequence["JsonValue"] | Mapping[str, "JsonValue"]
)
JsonObject = Mapping[str, JsonValue]


def func3(json_object: JsonObject) -> None:
    match json_object:
        case {
            "a": {
                "b": [
                    {
                        "c": "d",
                    }
                ],
            }
        }:
            pass


TA1 = tuple[Literal["a", "b", "c"], int]


def func4(vals: list[str]) -> TA1:
    x: TA1 = ("c", 0)

    for val in vals:
        match x[0]:
            case "b":
                if val.startswith("x"):
                    x = ("a", 1)
                continue
            case "c":
                if val.startswith("y"):
                    x = ("b", 2)
                continue
            case _:
                pass
    return x



# This sample tests special forms that are not compatible with type[T],
# type[object] or type[Any].

from typing import (
    Annotated,
    Any,
    Callable,
    ClassVar,
    Concatenate,
    Final,
    Generic,
    Literal,
    LiteralString,
    NotRequired,
    Optional,
    Protocol,
    Required,
    Self,
    TypeAlias,
    TypeGuard,
    Union,
    Unpack,
)
from typing_extensions import ReadOnly  # pyright: ignore[reportMissingModuleSource]

# The following should all generate an error.
Literal()
Literal[0]()
ClassVar()
Unpack()
Required()
NotRequired()
Protocol()
Generic()
Final()
Callable()
Callable[..., Any]()
Union()
Optional()
LiteralString()
Self()
Concatenate()
TypeAlias()
TypeGuard()
Annotated()
ReadOnly()
Union[int, str]()
(int | str)()



# This sample tests the case where a generic type uses a ParamSpec
# as a type parameter and it is specialized using an empty signature.

from typing import Any, Callable, Concatenate, Generic, ParamSpec

P = ParamSpec("P")


class Context: ...


CommandHandler = Callable[Concatenate[Context, P], Any]


class Command(Generic[P]):
    def __init__(self, handler: CommandHandler[P]) -> None: ...


def handler_no_args(ctx: Context) -> None: ...


def handler_one_arg(ctx: Context, a: int) -> None: ...


cmd_no_args = Command(handler_no_args)
reveal_type(cmd_no_args, expected_text="Command[()]")

cmd_one_arg = Command(handler_one_arg)
reveal_type(cmd_one_arg, expected_text="Command[(a: int)]")
